/* ###################################################################
**     This component module is generated by Processor Expert. Do not modify it.
**     Filename    : I2C.c
**     Project     : ChargeurL200
**     Processor   : MC9S08SH8MPJ
**     Component   : InternalI2C
**     Version     : Component 01.287, Driver 01.28, CPU db: 3.00.067
**     Compiler    : CodeWarrior HCS08 C Compiler
**     Date/Time   : 2019-11-30, 12:30, # CodeGen: 19
**     Abstract    :
**          This component encapsulates the internal I2C communication 
**          interface. The implementation of the interface is based 
**          on the Philips I2C-bus specification version 2.0. 
**          Interface features:
**          MASTER mode
**            - Multi master communication
**            - The combined format of communication possible 
**              (see "Automatic stop condition" property)
**            - 7-bit slave addressing (10-bit addressing can be made as well)
**            - Acknowledge polling provided
**            - No wait state initiated when a slave device holds the SCL line low
**            - Holding of the SCL line low by slave device recognized as 'not available bus'
**            - Invalid start/stop condition detection provided
**          SLAVE mode
**            - 7-bit slave addressing
**            - General call address detection provided
**     Settings    :
**         Serial channel              : IIC
**
**         Protocol
**             Mode                    : MASTER - SLAVE
**             Slave address           : 50
**             Empty character         : 0
**             Auto stop condition     : yes
**             SCL frequency           : 100 kHz
**
**         Initialization
**
**             Target slave address    : 8
**             Component init          : Enabled
**             Events                  : Enabled
**
**         Registers
**             Input buffer            : IICD      [$005C]
**             Output buffer           : IICD      [$005C]
**             Control register        : IICC1     [$005A]
**             Status register         : IICS      [$005B]
**             Baud setting reg.       : IICF      [$0059]
**             Address register        : IICA      [$0058]
**
**         Interrupt
**             Vector name             : Viic
**             Priority                : 
**
**         Used pins                   :
**       ----------------------------------------------------------
**            Function    | On package |    Name
**       ----------------------------------------------------------
**              SDA       |     6      |  PTB6_SDA_XTAL
**              SCL       |     5      |  PTB7_SCL_EXTAL
**       ----------------------------------------------------------
**     Contents    :
**         SendChar        - byte I2C_SendChar(byte Chr);
**         RecvChar        - byte I2C_RecvChar(byte *Chr);
**         SendBlock       - byte I2C_SendBlock(void* Ptr, word Siz, word *Snt);
**         RecvBlock       - byte I2C_RecvBlock(void* Ptr, word Siz, word *Rcv);
**         ClearTxBuf      - byte I2C_ClearTxBuf(void);
**         ClearRxBuf      - byte I2C_ClearRxBuf(void);
**         GetCharsInTxBuf - word I2C_GetCharsInTxBuf(void);
**         GetCharsInRxBuf - word I2C_GetCharsInRxBuf(void);
**         SelectSlave     - byte I2C_SelectSlave(byte Slv);
**         SetMode         - byte I2C_SetMode(bool MasterMode);
**
**     Copyright : 1997 - 2014 Freescale Semiconductor, Inc. 
**     All Rights Reserved.
**     
**     Redistribution and use in source and binary forms, with or without modification,
**     are permitted provided that the following conditions are met:
**     
**     o Redistributions of source code must retain the above copyright notice, this list
**       of conditions and the following disclaimer.
**     
**     o Redistributions in binary form must reproduce the above copyright notice, this
**       list of conditions and the following disclaimer in the documentation and/or
**       other materials provided with the distribution.
**     
**     o Neither the name of Freescale Semiconductor, Inc. nor the names of its
**       contributors may be used to endorse or promote products derived from this
**       software without specific prior written permission.
**     
**     THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
**     ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
**     WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
**     DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
**     ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
**     (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
**     LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
**     ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
**     (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
**     SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**     
**     http: www.freescale.com
**     mail: support@freescale.com
** ###################################################################*/
/*!
** @file I2C.c
** @version 01.28
** @brief
**          This component encapsulates the internal I2C communication 
**          interface. The implementation of the interface is based 
**          on the Philips I2C-bus specification version 2.0. 
**          Interface features:
**          MASTER mode
**            - Multi master communication
**            - The combined format of communication possible 
**              (see "Automatic stop condition" property)
**            - 7-bit slave addressing (10-bit addressing can be made as well)
**            - Acknowledge polling provided
**            - No wait state initiated when a slave device holds the SCL line low
**            - Holding of the SCL line low by slave device recognized as 'not available bus'
**            - Invalid start/stop condition detection provided
**          SLAVE mode
**            - 7-bit slave addressing
**            - General call address detection provided
*/         
/*!
**  @addtogroup I2C_module I2C module documentation
**  @{
*/         


/* MODULE I2C. */


#pragma MESSAGE DISABLE C4002          /* Disable warning C4002 "Result not used" */

#include "Events.h"
#include "I2C.h"

/* SerFlag bits */
#define OVERRUN_ERR      0x01U         /* Overrun error flag bit   */
#define WAIT_RX_CHAR     0x02U         /* Wait for received char. flag bit (Master)  */
#define CHAR_IN_TX       0x04U         /* Char is in TX buffer (Master)    */
#define CHAR_IN_RX       0x08U         /* Char is in RX buffer     */
#define FULL_TX          0x10U         /* Full transmit buffer     */
#define IN_PROGRES       0x20U         /* Communication is in progress (Master) */
#define FULL_RX          0x40U         /* Full receive buffer      */
#define MSxSL            0x80U         /* Master x Slave flag bit  */

/* Mask definitions used in 'MainComm' internal method */

static byte I2C_SlaveAddr;             /* Variable for Slave address */
static word InpLenM;                   /* Length of input bufer's content */
static byte *InpPtrM;                  /* Pointer to input buffer for Master mode */
static word OutLenM;                   /* Length of output bufer's content */
static byte * OutPtrM;                 /* Pointer to output buffer for Master mode */
static byte InpLenS;                   /* Length of input buffer's content */
static byte InpPtrRS;                  /* Index for reading from input buffer */
static byte InpPtrWS;                  /* Index for writing to input buffer */
static byte InpBufferS[8];             /* Input buffer I2C commmunication */
static byte OutLenS;                   /* Length of output bufer's content */
static byte OutPtrRS;                  /* Index for reading from output buffer */
static byte OutPtrWS;                  /* Index for writing to output buffer */
static byte OutBufferS[8];             /* Output buffer for I2C commmunication */
word I2C_SndRcvTemp;                   /* Temporary variable for SendChar (RecvChar) when they call SendBlock (RecvBlock) */
static byte ChrTemp;                   /* Temporary variable for SendChar method */
static byte I2C_SerFlag;               /* Flags for serial communication */
                                       /* Bits: 0 - OverRun error */
                                       /*       1 - Wait for received char. flag bit (Master) */
                                       /*       2 - Char is in TX buffer (Master) */
                                       /*       3 - Char in RX buffer */
                                       /*       4 - Full TX buffer */
                                       /*       5 - Running int from TX */
                                       /*       6 - Full RX buffer */
                                       /*       7 - Master x Slave */
/*
** ===================================================================
**     Method      :  I2C_Interrupt (component InternalI2C)
**
**     Description :
**         The method services the interrupt of the selected peripheral(s)
**         and eventually invokes event(s) of the component.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
#define RXAK 0x01U
#define SRW  0x04U
#define IBAL 0x10U
#define IAAS 0x40U

#define ON_ARBIT_LOST 0x01U
#define ON_FULL_RX    0x02U
#define ON_RX_CHAR    0x04U
#define ON_FREE_TX    0x08U
#define ON_TX_CHAR    0x10U
#define ON_OVERRUN    0x20U
#define ON_TX_EMPTY   0x40U

ISR(I2C_Interrupt)
{
  byte Flags = 0U;                     /* Temporary variable for flags */
  byte Status;                         /* Safe status register */

  Status = IICS;
  IICS_IICIF = 1U;                     /* Clear interrupt flag and ARBL flag if set (by means of the read modify write effect) */
  if(IICC1_MST) {                      /* Is device in master mode? */
    if(IICC1_TX) {                     /* Is device in Tx mode? */
      if(Status & RXAK) {              /* NACK received? */
        IICC1_MST = 0U;                /* Switch device to slave mode (stop signal sent) */
        IICC1_TX = 0U;                 /* Switch to Rx mode */
        OutLenM = 0U;                  /* No character for sending */
        InpLenM = 0U;                  /* No character for reception */
        I2C_SerFlag &= (uint8_t)~(uint8_t)((CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES)); /* No character for sending or reception*/
      }
      else {
        if(OutLenM) {                  /* Is any char. for transmitting? */
          OutLenM--;                   /* Decrease number of chars for the transmit */
          IICD = *(OutPtrM)++;         /* Send character */
        }
        else {
          if(InpLenM) {                /* Is any char. for reception? */
            if(InpLenM == 1U) {        /* If only one char to receive */
              IICC1_TXAK = 1U;         /* then transmit ACK disable */
            }
            else {
              IICC1_TXAK = 0U;         /* else transmit ACK enable */
            }
            IICC1_TX = 0U;             /* Switch to Rx mode */
            (void)IICD;                /* Dummy read character */
          }
          else {
            I2C_SerFlag &= (uint8_t)~(uint8_t)(IN_PROGRES); /* Clear flag "busy" */
            IICC1_MST = 0U;            /* Switch device to slave mode (stop signal sent) */
            IICC1_TX = 0U;             /* Switch to Rx mode */
            I2C_OnTransmitData();      /* Invoke OnTransmitData event */
          }
        }
      }
    }
    else {
      InpLenM--;                       /* Decrease number of chars for the receive */
      if(InpLenM) {                    /* Is any char. for reception? */
        if(InpLenM == 1U) {
          IICC1_TXAK = 1U;             /* Transmit ACK disable */
        }
      }
      else {
        IICC1_MST = 0U;                /* If no, switch device to slave mode (stop signal sent) */
        IICC1_TXAK = 0U;               /* Transmit ACK enable */
      }
      *(InpPtrM)++ = IICD;             /* Receive character */
      if(!InpLenM) {                   /* Is any char. for reception? */
        I2C_OnReceiveData();           /* Invoke OnReceiveData event */
      }
    }
  }
  else {
    if(Status & IAAS) {                /* Addressed as a slave? */
      if(Status & IBAL) {              /* Arbitration lost? */
        OutLenM = 0U;                  /* No character for sending */
        InpLenM = 0U;                  /* No character for reception */
        I2C_SerFlag &= (uint8_t)~(uint8_t)((CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES)); /* No character for sending or reception*/
        IICC1_TX = 0U;                 /* Switch to Rx mode */
      }
      if(Status & SRW) {               /* Read? */
        IICC1_TX = 1U;                 /* Switch to Tx mode */
        if (OutLenS) {                 /* Is number of bytes in the transmit buffer greater then 0? */
          OutLenS--;                   /* Decrease number of chars in the transmit buffer */
          IICD = OutBufferS[OutPtrRS++]; /* Store char to transmitter register */
          if(OutPtrRS >= 8U) {         /* Is the index out of the transmit buffer? */
            OutPtrRS = 0U;             /* Set index on the first item into the transmit buffer */
          }
          Flags |= ON_TX_CHAR;         /* Set OnTxChar flag */
        }
        else {
          IICD = I2C_EOF;              /* Store the empty char to the transmit register */
        }
      }
      else{
        IICC1_TX = 0U;                 /* Switch to Rx mode */
        (void)IICD;                    /* Dummy read character */
      }
    }
    else {
      if(Status & IBAL) {              /* Arbitration lost? */
        OutLenM = 0U;                  /* No character for sending */
        InpLenM = 0U;                  /* No character for reception */
        I2C_SerFlag &= (uint8_t)~(uint8_t)((CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES)); /* No character for sending or reception*/
        IICC1_TX = 0U;                 /* Switch to Rx mode */
      }
      else {
        if(IICC1_TX) {                 /* Transmit? */
          if(Status & RXAK) {          /* NACK from receiver */
            IICC1_TX = 0U;             /* Switch to Rx mode */
            (void)IICD;                /* Dummy read character */
          }
          else {
            if (OutLenS) {             /* Is number of bytes in the transmit buffer greater then 0? */
              OutLenS--;               /* Decrease number of chars in the transmit buffer */
              IICD = OutBufferS[OutPtrRS++]; /* Store char to transmitter register */
              if(OutPtrRS >= 8U) {     /* Is the index out of the transmit buffer? */
                OutPtrRS = 0U;         /* Set index on the first item into the transmit buffer */
              }
              Flags |= ON_TX_CHAR;     /* Set OnTxChar flag */
            }
            else {
              IICD = I2C_EOF;          /* Store the empty char to the transmit register */
            }
          }
        }
        else {
          if(InpLenS < 8U) {           /* Is number of bytes in the receive buffer lower than size of buffer? */
            InpLenS++;                 /* Increse number of chars in the receive buffer */
            InpBufferS[InpPtrWS++] = IICD; /* Save received char to the receive buffer */
            if(InpPtrWS >= 8U) {       /* Is the index out of the receive buffer? */
              InpPtrWS = 0U;           /* Set index on the first item into the receive buffer */
            }
            Flags |= ON_RX_CHAR;       /* If yes then set the OnRxChar flag */
          }
          else {
            (void)IICD;                /* Dummy read data register */
            I2C_SerFlag |= FULL_RX;    /* Set flag "full RX buffer" */
          }
        }
      }
    }
    if(Flags & ON_RX_CHAR) {           /* Is OnRxChar flag set? */
      I2C_OnRxChar();                  /* If yes then invoke user event */
    }
    if (Flags & ON_TX_CHAR) {          /* Is OnTxChar flag set? */
      I2C_OnTxChar();                  /* If yes then invoke user event */
    }
  }
}

/*
** ===================================================================
**     Method      :  I2C_SendChar (component InternalI2C)
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 0 to the I2C bus and then writes one
**         character (byte) to the bus. The slave address must be
**         specified before, by the "SelectSlave" or "SelectSlave10"
**         method or in the component initialization section, "Target
**         slave address init" property. If interrupt service is
**         enabled and the method returns ERR_OK, it doesn't mean that
**         transmission was successful. The state of transmission is
**         obtainable from (OnTransmitData, OnError or OnArbitLost)
**         events. 
**         When working as a SLAVE, this method writes a character to
**         the internal output slave buffer and, after the master
**         starts the communication, to the I2C bus. If no character is
**         ready for a transmission (internal output slave buffer is
**         empty), the Empty character will be sent (see "Empty
**         character" property).
**     Parameters  :
**         NAME            - DESCRIPTION
**         Chr             - Character to send.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by an acknowledge (only in
**                           master mode and when interrupt service is
**                           disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot transmit data
**                           ERR_TXFULL - Transmitter is full (slave
**                           mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
** ===================================================================
*/
byte I2C_SendChar(byte Chr)
{
  if(I2C_SerFlag & MSxSL){
    if((IICS_BUSY) || (InpLenM) || (I2C_SerFlag & (CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES))) { /* Is the bus busy */
      return ERR_BUSOFF;               /* If yes then error */
    }
  }
  ChrTemp = Chr;                       /* Save character */
  return (I2C_SendBlock(&ChrTemp, 1U, &I2C_SndRcvTemp)); /* Send character and return */
}

/*
** ===================================================================
**     Method      :  I2C_RecvChar (component InternalI2C)
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 1 to the I2C bus, then reads one
**         character (byte) from the bus and then sends the stop
**         condition. The slave address must be specified before, by
**         the "SelectSlave" or "SelectSlave10" method or in component
**         initialization section, property "Target slave address init".
**         If interrupt service is enabled and the method returns
**         ERR_OK, it doesn't mean that transmission was finished
**         successfully. The state of transmission must be tested by
**         means of events (OnReceiveData, OnError or OnArbitLost). In
**         case of successful transmission, received data is ready
**         after OnReceiveData event is called. 
**         When working as a SLAVE, this method reads a character from
**         the input slave buffer.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Chr             - Received character.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by the acknowledge (only
**                           in master mode and when interrupt service
**                           is disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot receive data
**                           ERR_RXEMPTY - No data in receiver (slave
**                           mode only)
**                           ERR_OVERRUN - Overrun error was detected
**                           from the last character or block received
**                           (slave mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
**                           ERR_NOTAVAIL - Method is not available in
**                           current mode - see the comment in the
**                           generated code
** ===================================================================
*/
/*
byte I2C_RecvChar(byte *Chr)

**  This method is implemented as a macro. See I2C.h file.  **
*/

/*
** ===================================================================
**     Method      :  I2C_SendBlock (component InternalI2C)
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 0 to the I2C bus and then writes the
**         block of characters to the bus. The slave address must be
**         specified before, by the "SelectSlave" or "SlaveSelect10"
**         method or in component initialization section, "Target slave
**         address init" property. If interrupt service is enabled and
**         the method returns ERR_OK, it doesn't mean that transmission
**         was successful. The state of transmission is detectable by
**         means of events (OnTransmitData, OnError or OnArbitLost).
**         Data to be send is not copied to an internal buffer and
**         remains in the original location. Therefore the content of
**         the buffer should not be changed until the transmission is
**         complete. Event OnTransmitData can be used to detect the end
**         of the transmission.
**         When working as a SLAVE, this method writes a block of
**         characters to the internal output slave buffer and then,
**         after the master starts the communication, to the I2C bus.
**         If no character is ready for a transmission (internal output
**         slave buffer is empty), the "Empty character" will be sent
**         (see "Empty character" property). In SLAVE mode the data are
**         copied to an internal buffer, if specified by "Output buffer
**         size" property.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Ptr             - Pointer to the block of data to send.
**         Siz             - Size of the block.
**       * Snt             - Amount of data sent (moved to a buffer).
**                           In master mode, if interrupt support is
**                           enabled, the parameter always returns the
**                           same value as the parameter 'Siz' of this
**                           method.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by the acknowledge (only
**                           in master mode and when interrupt service
**                           is disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot transmit data
**                           ERR_TXFULL - Transmitter is full. Some data
**                           has not been sent. (slave mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
** ===================================================================
*/
byte I2C_SendBlock(const void * Ptr,word Siz,word *Snt)
{
  word count;                          /* Number of sent chars */

  if (!Siz) {                          /* Test variable Size on zero */
    *Snt = 0U;
    return ERR_OK;                     /* If zero then OK */
  }
  if(I2C_SerFlag & MSxSL){
    if((IICS_BUSY) || (InpLenM) || (I2C_SerFlag & (CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES))) { /* Is the bus busy */
      return ERR_BUSOFF;               /* If yes then error */
    }
    EnterCritical();                   /* Enter the critical section */
    I2C_SerFlag |= IN_PROGRES;         /* Set flag "busy" */
    OutLenM = Siz;                     /* Set lenght of data */
    OutPtrM = (byte *)Ptr;             /* Save pointer to data for transmitting */
    IICC1_TX = 1U;                     /* Set TX mode */
    if(IICC1_MST) {                    /* Is device in master mode? */
      IICC1_RSTA = 1U;                 /* If yes then repeat start cycle generated */
    }
    else {
      IICC1_MST = 1U;                  /* If no then start signal generated */
    }
    IICD = I2C_SlaveAddr;              /* Send slave address */
    ExitCritical();                    /* Exit the critical section */
    *Snt = Siz;                        /* Dummy number of really sent chars */
  }
  else {
    EnterCritical();                   /* Enter the critical section */
    *Snt = 0U;                         /* Return number of sent chars */
    for(count=0U; count < Siz; count++) {
      if(OutLenS < 8U) {               /* Is number of bytes in the transmit buffer lower than size of buffer? */
        OutLenS++;                     /* Increase number of bytes in the transmit buffer */
        OutBufferS[OutPtrWS++] = *((byte *)Ptr + count); /* Store char to buffer */
        if (OutPtrWS >= 8U) {          /* Is the pointer out of the transmit buffer? */
          OutPtrWS = 0U;               /* Set pointer to the first item in the transmit buffer */
        }
        (*Snt)++;                      /* Increment number of sent chars */
      }
      else {
        ExitCritical();                /* Exit the critical section */
        return ERR_TXFULL;             /* Return with error */
      }
    }
    ExitCritical();                    /* Exit the critical section */
  }
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  I2C_RecvBlock (component InternalI2C)
**     Description :
**         When working as a MASTER, this method writes one (7-bit
**         addressing) or two (10-bit addressing) slave address bytes
**         inclusive of R/W bit = 1 to the I2C bus, then reads the
**         block of characters from the bus and then sends the stop
**         condition. The slave address must be specified before, by
**         the "SelectSlave" or "SelectSlave10" method or in component
**         initialization section, "Target slave address init" property.
**         If interrupt service is enabled and the method returns
**         ERR_OK, it doesn't mean that transmission was finished
**         successfully. The state of transmission must be tested by
**         means of events (OnReceiveData, OnError or OnArbitLost). In
**         case of successful transmission, received data is ready
**         after OnReceiveData event is called. 
**         When working as a SLAVE, this method reads a block of
**         characters from the input slave buffer.
**     Parameters  :
**         NAME            - DESCRIPTION
**       * Ptr             - A pointer to the block space for received
**                           data.
**         Siz             - The size of the block.
**       * Rcv             - Amount of received data. In master mode,
**                           if interrupt support is enabled, the
**                           parameter always returns the same value as
**                           the parameter 'Siz' of this method.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  Device is disabled
**                           ERR_BUSY - The slave device is busy, it
**                           does not respond by an acknowledge (only in
**                           master mode and when interrupt service is
**                           disabled)
**                           ERR_BUSOFF - Clock timeout elapsed or
**                           device cannot receive data
**                           ERR_RXEMPTY - The receive buffer didn't
**                           contain the requested number of data. Only
**                           available data (or no data) has been
**                           returned  (slave mode only).
**                           ERR_OVERRUN - Overrun error was detected
**                           from last character or block receiving
**                           (slave mode only)
**                           ERR_ARBITR - Arbitration lost (only when
**                           interrupt service is disabled and in master
**                           mode)
**                           ERR_NOTAVAIL - Method is not available in
**                           current mode - see the comment in the
**                           generated code.
** ===================================================================
*/
byte I2C_RecvBlock(void* Ptr,word Siz,word *Rcv)
{
  word count;                          /* Number of sent chars */
  byte errrc = ERR_OK;                 /* Most serious error */

  if (!Siz) {                          /* Test variable Size on zero */
    *Rcv = 0U;
    return ERR_OK;                     /* If zero then OK */
  }
  if(I2C_SerFlag & MSxSL){
    if((IICS_BUSY) || (InpLenM) || (I2C_SerFlag & (CHAR_IN_TX|WAIT_RX_CHAR|IN_PROGRES))) { /* Is the bus busy */
      return ERR_BUSOFF;               /* If yes then error */
    }
    EnterCritical();                   /* Enter the critical section */
    InpLenM = Siz;                     /* Set lenght of data */
    InpPtrM = (byte *)Ptr;             /* Save pointer to data for reception */
    IICC1_TX = 1U;                     /* Set TX mode */
    if(IICC1_MST) {                    /* Is device in master mode? */
      IICC1_RSTA = 1U;                 /* If yes then repeat start cycle generated */
    }
    else {
      IICC1_MST = 1U;                  /* If no then start signal generated */
    }
    IICD = (byte)(I2C_SlaveAddr+1U);   /* Send slave address */
    ExitCritical();                    /* Exit the critical section */
    *Rcv = Siz;                        /* Dummy number of really received chars */
  }
  else {
    EnterCritical();                   /* Enter the critical section */
    if ((I2C_SerFlag & (OVERRUN_ERR | FULL_RX)) != 0U) { /* Is the overrun occured? */
      errrc = ERR_OVERRUN;             /* If yes then return OVERRUN error */
    }
    I2C_SerFlag &= (uint8_t)~(uint8_t)((OVERRUN_ERR | CHAR_IN_RX | FULL_RX)); /* Clear flag "char in RX buffer" */
    *Rcv = 0U;                         /* Clear number of received chars */
    for(count=0U; count < Siz; count++) {
      if(InpLenS) {                    /* Is number of bytes in the receive buffer lower than size of buffer? */
        InpLenS--;                     /* Decrease number of received chars */
        *((byte *)Ptr + count) = InpBufferS[InpPtrRS++]; /* Read the char */
        if (InpPtrRS >= 8U) {          /* Is the index out of the receive buffer? */
          InpPtrRS = 0U;               /* Set index to the first item into the receive buffer */
        }
        (*Rcv)++;                      /* Increment number of sent chars */
      }
      else {
        ExitCritical();                /* Exit the critical section */
        if(errrc==ERR_OVERRUN) {
          return ERR_OVERRUN;          /* Return with error */
        }
        else {
          return ERR_RXEMPTY;          /* Return with error */
        }
      }
    }
    ExitCritical();                    /* Exit the critical section */
    return errrc;                      /* Return error code */
  }
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  I2C_ClearTxBuf (component InternalI2C)
**     Description :
**         This method clears all characters in internal slave output
**         buffer. This method is not available for the MASTER mode.
**     Parameters  : None
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
byte I2C_ClearTxBuf(void)
{
  EnterCritical();                     /* Enter the critical section */
  OutLenS = 0U;                        /* Set number of chars in the receive buffer to 0 */
  OutPtrRS = 0U;                       /* Set indexes on the first item in the receive buffer */
  OutPtrWS = 0U;
  I2C_SerFlag &= (uint8_t)~(uint8_t)((FULL_TX|CHAR_IN_TX));
  ExitCritical();                      /* Exit the critical section */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  I2C_ClearRxBuf (component InternalI2C)
**     Description :
**         This method clears all characters in internal slave input
**         buffer. This method is not available for the MASTER mode.
**     Parameters  : None
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
byte I2C_ClearRxBuf(void)
{
  EnterCritical();                     /* Enter the critical section */
  InpLenS = 0U;                        /* Set number of chars in the transmit buffer to 0 */
  InpPtrRS = 0U;                       /* Set indexes on the first item in the transmit buffer */
  InpPtrWS = 0U;
  I2C_SerFlag &= (uint8_t)~(uint8_t)((OVERRUN_ERR | CHAR_IN_RX | FULL_RX));
  ExitCritical();                      /* Exit the critical section */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  I2C_GetCharsInTxBuf (component InternalI2C)
**     Description :
**         Returns number of characters in the output buffer. In SLAVE
**         mode returns the number of characters in the internal slave
**         output buffer. In MASTER mode returns number of characters
**         to be sent from the user buffer (passed by SendBlock method).
**         This method is not supported in polling mode.
**     Parameters  : None
**     Returns     :
**         ---             - Number of characters in the output buffer.
** ===================================================================
*/
word I2C_GetCharsInTxBuf(void)
{
  if (I2C_SerFlag & MSxSL) {
    return(OutLenM);                   /* Return number of chars remaining in the Master Tx buffer */
  }
  else  {
    return((word)OutLenS);             /* Return number of chars in the transmit buffer */
  }
}

/*
** ===================================================================
**     Method      :  I2C_GetCharsInRxBuf (component InternalI2C)
**     Description :
**         Returns number of characters in the input buffer. In SLAVE
**         mode returns the number of characters in the internal slave
**         input buffer. In MASTER mode returns number of characters to
**         be received into a user buffer (passed by RecvChar or
**         RecvBlock method).
**         This method is not supported in polling mode.
**     Parameters  : None
**     Returns     :
**         ---             - Number of characters in the input buffer.
** ===================================================================
*/
word I2C_GetCharsInRxBuf(void)
{
  if(I2C_SerFlag & MSxSL) {
    return(InpLenM);                   /* Return number of chars remaining in the Master Tx buffer */
  }
  else {
    return((word)InpLenS);             /* Return number of chars in receive buffer */
  }
}

/*
** ===================================================================
**     Method      :  I2C_SelectSlave (component InternalI2C)
**     Description :
**         This method selects a new slave for communication by its
**         7-bit slave address value. Any send or receive method
**         directs to or from selected device, until a new slave device
**         is selected by this method. This method is not available for
**         the SLAVE mode.
**     Parameters  :
**         NAME            - DESCRIPTION
**         Slv             - 7-bit slave address value.
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_BUSY - The device is busy, wait until
**                           the current operation is finished.
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
**                           ERR_DISABLED -  The device is disabled
** ===================================================================
*/
byte I2C_SelectSlave(byte Slv)
{
  if (IICC1_MST == 1U) {               /* Is the device in the active state? */
    return ERR_BUSY;                   /* If yes then error */
  }
  I2C_SlaveAddr = (byte)(Slv << 1);    /* Set slave address */
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  I2C_SetMode (component InternalI2C)
**     Description :
**         This method sets actual operating mode of this component.
**         This method is available only for the MASTER - SLAVE mode.
**         Note: If MASTER mode is selected as an operating mode, the
**         device can still be selected as a slave and receive/transmit
**         data. However, it's necessary to select slave operating mode
**         using SetMode( FALSE ) to access this data.
**     Parameters  :
**         NAME            - DESCRIPTION
**         MasterMode      - Operating mode value:
**                           TRUE - Master
**                           FALSE - Slave
**     Returns     :
**         ---             - Error code, possible codes:
**                           ERR_OK - OK
**                           ERR_SPEED - This device does not work in
**                           the active speed mode
** ===================================================================
*/
byte I2C_SetMode(bool MasterMode)
{
  I2C_SerFlag &= (uint8_t)~(uint8_t)(MSxSL); /* Set mode */
  I2C_SerFlag |= (uint8_t)((MasterMode)?MSxSL:0U);
  return ERR_OK;                       /* OK */
}

/*
** ===================================================================
**     Method      :  I2C_Init (component InternalI2C)
**
**     Description :
**         Initializes the associated peripheral(s) and the component 
**         internal variables. The method is called automatically as a 
**         part of the application initialization code.
**         This method is internal. It is used by Processor Expert only.
** ===================================================================
*/
void I2C_Init(void)
{
  I2C_SerFlag = 0x80U;                 /* Reset all flags */
  I2C_SlaveAddr = 0x10U;               /* Set variable for slave address */
  InpLenM = 0U;                        /* No data to be received */
  InpLenS = 0U;                        /* No char in the receive buffer */
  InpPtrWS = 0U;                       /* Set indexes on the first item in the receive buffer */
  InpPtrRS = 0U;
  OutLenS = 0U;                        /* No char in the transmit buffer */
  OutPtrWS = 0U;                       /* Set indexes on the first item in the transmit buffer */
  OutPtrRS = 0U;
  /* IICA: AD7=0,AD6=1,AD5=1,AD4=0,AD3=0,AD2=1,AD1=0,??=0 */
  IICA = 0x64U;                        /* Define slave address */
  /* IICC2: GCAEN=0,ADEXT=0,??=0,??=0,??=0,AD10=0,AD9=0,AD8=0 */
  IICC2 = 0x00U;
  /* IICF: MULT1=0,MULT0=1,ICR5=0,ICR4=0,ICR3=0,ICR2=1,ICR1=1,ICR0=1 */
  setReg8(IICF, 0x47U);                 
  IICC1_IICEN = 1U;                    /* Enable device */
  /* IICC1: IICEN=1,IICIE=1,MST=0,TX=0,TXAK=0,RSTA=0,??=0,??=0 */
  IICC1 = 0xC0U;                       /* Control register settings */
}


/* END I2C. */

/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.3 [05.09]
**     for the Freescale HCS08 series of microcontrollers.
**
** ###################################################################
*/
