/*
 * AffI2C_RW1063.c
 *
 *  Created on: Jul 3, 2019
 *      Author: Dominique
 */
#include "I2C.h"
#include "AffI2C_RW1063.h"
// Four 7-bit slave addresses (0111100, 0111101, 0111110 and 0111111) are reserved for the RW1063. The least
// significant bit of the slave address is set by connecting the input SA0 (DB0) and SA1 (DB1) to either logic 0
// (VSS) or logic 1 (VDD).
#define I2C_ADDRESS 0x3C //0b0111100
// #define I2C_ADDRESS 0x3D //0b0111101
// #define I2C_ADDRESS 0x3E //0b0111110
// #define I2C_ADDRESS 0x3F //0b0111111

//============================================================================
void writeCommand(uint8_t command)
{
	uint8_t datatosend[2];
	uint8_t datatransmit[2];
//Define slave address
	I2C_SelectSlave(I2C_ADDRESS);  
  
//Send the command via I2C:
	datatosend[0] = 0;
	datatosend[1] = command;
	I2C_SendBlock(datatosend,2,datatransmit);
    LCD_Delay_ms(1);
}
//============================================================================
void writeChar(uint8_t data)
{
	uint8_t datatosend[2];
	uint8_t datatransmit[2];
//Define slave address
	I2C_SelectSlave(I2C_ADDRESS);  
  
//Send the command via I2C:
	datatosend[0] = 0x40;		//Just one data
	datatosend[1] = data;
	I2C_SendBlock(datatosend,2,datatransmit);
    LCD_Delay_ms(1);
}
//============================================================================
void write_msg(char *msg){
	while(*msg!=0){
		writeChar(*msg);
		msg++;
	}
}
//============================================================================
void clrscr (void){
	writeCommand(0x01);
	LCD_Delay_ms(1);
}
//============================================================================
void locate(uint8_t column, uint8_t row){		// only two line (row = 0 or 1) and 16 column from 0 to 15
	if(row == 0) writeCommand(0x80+column);
	else writeCommand(0xC0+column);
	LCD_Delay_ms(1);
}
//============================================================================
void init_LCD (void){
	LCD_Delay_ms(50);
	writeCommand(0x38); // Function set
	LCD_Delay_ms(5);
	writeCommand(0x38); // Function set
	LCD_Delay_ms(1);
	writeCommand(0x38); // Function set
	LCD_Delay_ms(1);
	writeCommand(0x08); // Display OFF
	LCD_Delay_ms(1);
	writeCommand(0x01); // Clear display
	LCD_Delay_ms(10);
	writeCommand(0x06); // Entry mode set
	LCD_Delay_ms(1);
	writeCommand(0x0C); // Display ON cursor off blink off
	LCD_Delay_ms(1);
}
//*************************************************************************************
//**************************************************************************
//* Une simple  fonction delay  (utilise par les routines du LCD)
//**************************************************************************
//* Argument en entre:
//* unsigned char time: dure approximative en millisecondes
//**************************************************************************
void LCD_Delay_ms (unsigned char time)
{
	unsigned int temp;
	for(;time;time--) for(temp=(BUS_CLOCK/23);temp;temp--);
}
