#include "IO_Map.h"


//*******************************************************************  
//*** Routine d'criture de donnes vers l'esclave
//*** Necessite:  - Pointeur pour tableau de donnes
//***             - Nombre d'octets  transmettre
//***			  - adresse de l'esclave en mode 8bits
//****************************************************************
void I2C_TX(unsigned char *I2C_DATA, unsigned char nbr_octet, unsigned char slave_adr){
	uint8_t Temp, index;
	/* Make sure bus is idle */
		while (I2C_S_BUSY == 1);
		/* Put module in master TX mode (generates START) */
		I2C_C1_TX = 1;				    // Trasmit mode select
	    I2C_C1_MST = 1;                 // Select Master Mode (Send Start Bit)
        for(Temp=0;Temp<3;Temp++);    //  tempo      
		/* Put target address into I2C Data I/O register (I2C_D) */
		I2C_D = slave_adr&0xFE;
		/* Wait for I2SR (bus busy) to be set */
		while (I2C_S_BUSY == 0);
	    /* Wait for address transfer to complete */
		while (!(I2C_S_IICIF));
		I2C_S_IICIF = 1;
		//while (!I2C_S_TCF);            
		/* Send the contents of the TX buffer */
		index = 0;
		while (nbr_octet > 0)
		{
			I2C_D = I2C_DATA[index++];
			nbr_octet--;
			/* Wait for transfer to complete */
			while (!(I2C_S_IICIF));
			I2C_S_IICIF = 1;
		}
		
		/* Restore module to it's idle (but active) state */
		I2C_C1 = 0x80;  // TX = 0, MST = 0 => send stop condition
		
}
//*******************************************************************  
//*** Routine de lecture de données depuis l'esclave
//*** Necessite:  - Pointeur pour tableau de données
//***             - Nombre d'octets à recevoir
//***			  - adresse de l'esclave en mode 8bits
//****************************************************************
void I2C_RX(unsigned char *I2C_DATA, unsigned char nbr_octet, unsigned char slave_adr){
	uint8_t Temp, index, dummy_read;
	/* Make sure bus is idle */
		while (I2C_S_BUSY == 1);
		/* Put module in master TX mode (generates START) */
		I2C_C1_TX = 1;				    // Transmit mode
	    I2C_C1_MST = 1;                 // Select Master Mode (Send Start Bit)
        for(Temp=0;Temp<3;Temp++);    // tempo      
		/* Put target address into I2C Data I/O register (I2C_D) */
		I2C_D = slave_adr|0x01;
		/* Wait for I2SR (bus busy) to be set */
		while (I2C_S_BUSY == 0);
	    /* Wait for address transfer to complete */
		while (!(I2C_S_IICIF));
		I2C_S_IICIF = 1;
		//while (!I2C_S_TCF);            
		/* Clear TX/RX bit in order to set receive mode */
		I2C_C1_TX = 0; 
		
		/* Dummy read of IBDR to signal the module is ready for the next byte */
		dummy_read = I2C_D;
		
		/* Receive data from slave */
		index = 0;
		while (nbr_octet > 0)
		{
			/* Wait for transfer to complete */
			while (!(I2C_S_IICIF));
			I2C_S_IICIF = 1;
			/* Check for second-to-last and last byte transmission.  After second-to-last
			   byte is received, it is required to disable the ACK bit in order to signal
			   to the slave that the last byte has been received.  The actual NAck does not
			   take place until after the last byte has been received. */
			if (nbr_octet == 2) 
			{
				/* Disable Acknowledge, generate STOP after next byte transfer */
				I2C_C1_TXAK = 1;
			}
			if(nbr_octet == 1) 
			{
				/* Generate STOP */
				I2C_C1_MST = 0;
			}
			
			/* Store received data in RX buffer */
			I2C_DATA[index++] = I2C_D;
			nbr_octet--;
		}
		
		/* Restore module to it's idle (but active) state */
		I2C_C1 = 0x80;  // TX = 0, MST = 0 => send stop condition
		
}
//*******************************************************************  
//*** Routine d'criture puis de lecture de donnes vers l'esclave
//*** avec condition de restart
//*** Necessite:  - Pointeur pour tableau de donnes  crire
//*** 			  - Pointeur pour tableau de donnes  lire
//***             - Nombre d'octets  crire
//***             - Nombre d'octets  lire
//***			  - adresse de l'esclave en mode 8bits
//****************************************************************
void I2C_RXTX(unsigned char *I2C_DATA, unsigned char *I2C_DATAin, unsigned char nbr_octet, unsigned char nbr_octetin, unsigned char slave_adr){
	uint8_t dummy_read, index;
	uint8_t Temp;
	/* Make sure bus is idle */
		while (I2C_S_BUSY == 1);
		/* Put module in master TX mode (generates START) */
		I2C_C1_TX = 1;				    // Trasmit mode select
	    I2C_C1_MST = 1;                 // Select Master Mode (Send Start Bit)
        for(Temp=0;Temp<3;Temp++);    //  tempo      
		/* Put target address into I2C Data I/O register (I2C_D) */
		I2C_D = slave_adr&0xFE;
		/* Wait for I2SR (bus busy) to be set */
		while (I2C_S_BUSY == 0);
	    /* Wait for address transfer to complete */
		while (!(I2C_S_IICIF));
		I2C_S_IICIF = 1;
		//while (!I2C_S_TCF);            
		/* Send the contents of the TX buffer */
		index = 0;
		while (nbr_octet > 0)
		{
			I2C_D = I2C_DATA[index++];
			nbr_octet--;
			/* Wait for transfer to complete */
			while (!(I2C_S_IICIF));
			I2C_S_IICIF = 1;
		}
		
		for (Temp=1000; Temp; Temp--);
		/* Set IBCR.RSTA and put in master RX mode */
		I2C_C1_RSTA = 1;
		//I2C_C1_TX = 0;
		/* Put target address into I2C Data I/O register (I2C_D) */
		I2C_D = slave_adr|0x01;
		
	    /* Wait for address transfer to complete */
		while (!(I2C_S_IICIF));
		I2C_S_IICIF = 1;
		//while (!I2C_S_TCF);            
		/* Clear TX/RX bit in order to set receive mode */
		I2C_C1_TX = 0; 
		
		/* Dummy read of IBDR to signal the module is ready for the next byte */
		dummy_read = I2C_D;
		
		/* Receive data from slave */
		index = 0;
		while (nbr_octetin > 0)
		{
			/* Wait for transfer to complete */
			while (!(I2C_S_IICIF));
			I2C_S_IICIF = 1;
			/* Check for second-to-last and last byte transmission.  After second-to-last
			   byte is received, it is required to disable the ACK bit in order to signal
			   to the slave that the last byte has been received.  The actual NAck does not
			   take place until after the last byte has been received. */
			if (nbr_octetin == 2) 
			{
				/* Disable Acknowledge, generate STOP after next byte transfer */
				I2C_C1_TXAK = 1;
			}
			if(nbr_octetin == 1) 
			{
				/* Generate STOP */
				I2C_C1_MST = 0;
			}
			
			/* Store received data in RX buffer */
			I2C_DATAin[index++] = I2C_D;
			nbr_octetin--;
		}
		
		/* Restore module to it's idle (but active) state */
		I2C_C1 = 0x80;  // TX = 0, MST = 0 => send stop condition
	}	
