/* ###################################################################
**     Filename    : main.c
**     Project     : ChargeurLT1375
**     Processor   : MC9S08PA16VWJ
**     Version     : Driver 01.12
**     Compiler    : CodeWarrior HCS08 C Compiler
**     Date/Time   : 2019-07-06, 13:29, # CodeGen: 0
**     Abstract    :
**         Main module.
**         This module contains user's application code.
**     Settings    :
**     Contents    :
**         No public methods
**
** ###################################################################*/
/*!
** @file main.c
** @version 01.12
** @brief
**         Main module.
**         This module contains user's application code.
*/         
/*!
**  @addtogroup main_module main module documentation
**  @{
*/         
/* MODULE main */


/* Including needed modules to compile this module/procedure */
#include "Cpu.h"
#include "Events.h"
#include "WDOG.h"
#include "SDWN.h"
#include "Charge.h"
#include "Ch_Ich.h"
#include "I2C.h"
#include "Timer_1ms.h"
#include "Tplus.h"
#include "Tmoins.h"
#include "Val.h"
/* Include shared modules, which are used for whole project */
#include "PE_Types.h"
#include "PE_Error.h"
#include "PE_Const.h"
#include "IO_Map.h"

/* User includes (#include below this line is not maintained by Processor Expert) */

#include "delai.h"
#include "ssd1306.h"
#include "INA219.h"
#include "Routine_Conv_Bin-ASCII.h"
uint8_t row, column, valid, Ah;
uint32_t millis, oldmillis;
uint16_t VBat=0, Vshunt=0;
uint16_t Icharge=0, power=0;
char chaine[6];
uint8_t valU[2];

void main(void)
{
  /* Write your local variable definition here */

  /*** Processor Expert internal initialization. DON'T REMOVE THIS CODE!!! ***/
  PE_low_level_init();
  /*** End of Processor Expert internal initialization.                    ***/
  SDWN_SetVal();
  init_time();
  Ah = 7;
  (void)Ch_Ich_SetDutyUS(2*Ah);
  INA219_init();
  ssd1306_init();
  /* Fill the display with digits */
  	for (row = 0; row < SSD1306_LCD_HEIGHT / 8; row++) {
  		ssd1306_set_cursor(row, 0);
  		for (column = 0; column < SSD1306_LCD_WIDTH / 6; column++) {
  			ssd1306_putc('0' + (column + row ) % 10);
  		}
  	}
  	Cpu_Delay100US(1000);
  	ssd1306_clear();
  row = ((SSD1306_LCD_HEIGHT / 8) - 4) / 2;
  column = ((SSD1306_LCD_WIDTH / 6) - 10) / 2;
  ssd1306_set_cursor(0, 0);
  ssd1306_puts("Chargeur Batterie");
  ssd1306_BIGputs(1, 32,"LT1513");
  ssd1306_set_cursor(3, 0);
  ssd1306_puts("Ichmax = 250mA");
  Cpu_Delay100US(5000);
  ssd1306_clear();
  for(;;) { 
	  VBat = getBusVoltage_raw();
	  valU[0] = (uint8_t)VBat;
	  valU[1] = (uint8_t)(VBat>>8);
	  if(VBat < 11500)Charge_ClrVal();
	  else Charge_SetVal();
	  if((millis - oldmillis) >= 5000){
		  Icharge = getCurrent_raw();
		  Vshunt = getShuntVoltage_raw();
		  power = getPower_raw();
		  convBinAscii(VBat, chaine);
		  ssd1306_set_cursor(0, 0);
		  ssd1306_puts("Vbat = ");
		  ssd1306_puts(chaine);
		  convBinAscii(Icharge, chaine);
		  ssd1306_set_cursor(2, 0);
		  ssd1306_puts("Ich = ");
		  ssd1306_puts(chaine);
		  convBinAscii((uint16_t)Vshunt, chaine);
		  ssd1306_set_cursor(4, 0);
		  ssd1306_puts("VShunt = ");
		  ssd1306_puts(chaine);
		  convBinAscii(power, chaine);
		  ssd1306_set_cursor(6, 0);
		  ssd1306_puts("Power = ");
		  ssd1306_puts(chaine);
		  oldmillis = millis; 
	  }
	if(I2C_S_IAAS == 1){	//The calling address matches the programmed primary slave address in the A1 register
		I2C_C1_MST = 0;         // Select slave Mode
		I2C_C1_TX = I2C_S_SRW;	// set Rx/Tx mode in accordance to received calling address byte and clear the IAAS bit
		if(I2C_S_SRW == 1){
			I2C_C1_TX = 1;
			I2C_D = valU[0];
		/* Wait for transfer to complete */
			while (!(I2C_S_IICIF));
			I2C_S_IICIF = 1;
			I2C_D = valU[1];
		/* Wait for transfer to complete */
			while (!(I2C_S_IICIF));
			I2C_S_IICIF = 1;
		}
		I2C_C1_MST = 1;     // Select Master Mode
	}
	if((Tmoins_GetVal()==0)||(Tplus_GetVal()==0)||(Val_GetVal()==0) ){
		ssd1306_clear();
		valid = 0;
		
		ssd1306_set_cursor(0, 0);
		ssd1306_puts("Config. chargeur");
		ssd1306_set_cursor(2, 0);
		ssd1306_puts("Batt. Cap.=   Ah ");
		do{
			ssd1306_set_cursor(2, 11);
			ssd1306_putc((Ah/10)+0x30);
			ssd1306_putc((Ah%10)+0x30);
			if(Tplus_GetVal()==0){
				while(Tplus_GetVal()==0);
				Ah++;
				if(Ah>=11)Ah = 1;
			}
			if(Tmoins_GetVal()==0){
				while(Tmoins_GetVal()==0);
				Ah--;
				if(Ah<=0)Ah = 11;
			}
			if(Val_GetVal()==0){
				while(Val_GetVal()==0);
				(void)Ch_Ich_SetDutyUS(2*Ah);
				valid = 1;
				ssd1306_clear();
			}
			
		}while(valid==0);
		
	}
  }

  /*** Don't write any code pass this line, or it will be deleted during code generation. ***/
  /*** RTOS startup code. Macro PEX_RTOS_START is defined by the RTOS component. DON'T MODIFY THIS CODE!!! ***/
  #ifdef PEX_RTOS_START
    PEX_RTOS_START();                  /* Startup of the selected RTOS. Macro is defined by the RTOS component. */
  #endif
  /*** End of RTOS startup code.  ***/
  /*** Processor Expert end of main routine. DON'T MODIFY THIS CODE!!! ***/
  for(;;){}
  /*** Processor Expert end of main routine. DON'T WRITE CODE BELOW!!! ***/
} /*** End of main routine. DO NOT MODIFY THIS TEXT!!! ***/

/* END main */
/*!
** @}
*/
/*
** ###################################################################
**
**     This file was created by Processor Expert 10.3 [05.09]
**     for the Freescale HCS08 series of microcontrollers.
**
** ###################################################################
*/
