
#include "ssd1306.h"



void ssd1306_write_cmd(uint8_t cmd)
{
	uint8_t buf[2];

	buf[0] = SSD1306_MODE_CMD;
	buf[1] = cmd;
	
	I2C_TX(buf, 2, SSD1306_I2C_ADDR);
		
}

void ssd1306_write_data(uint8_t data[], uint8_t len)
{
	
	uint8_t ind,lengh;
	uint8_t buf[20];
	
	buf[0] = SSD1306_MODE_DATA;
	for(ind=0; ind<len; ind++){
		buf[ind+1] = data[ind];
	}
	I2C_TX(buf, len+1, SSD1306_I2C_ADDR);
			
}
// *******************************
void ssd1306_init(void) {
	uint8_t i;
	uint8_t init_cmd_sequence[] = {
		SSD1306_CMD_DISPLAY_OFF,	/* switch display off */
		SSD1306_CMD_SET_CLK_DIV,	/* set clock divisor */
		0x80,
		SSD1306_CMD_SET_MUX,		/* set mux ratio */
		(SSD1306_LCD_HEIGHT-1),		/* 39 */
		SSD1306_CMD_SET_OFFSET,		/* set display offset */
		0x00,
		SSD1306_CMD_SET_START_LINE,	/* set start line */
		SSD1306_CMD_SET_CHARGEPUMP,	/* set charge pump */
		0x14,				/* enable charge pump */
		SSD1306_CMD_SET_COM_SCAN_INC,	/* set COM scan direction */
		SSD1306_CMD_SET_COM_PINS,	/* set COM pins configuratio */
		0x12,
		SSD1306_CMD_SET_CONTRAST,	/* set contrast */
		0xAF,
		SSD1306_CMD_SET_PRECHARGE,	/* set pre-charge */
		0x25,
		SSD1306_CMD_SET_VCOM_DESELECT,	/* set VCOM deselect level */
		0x20,
		SSD1306_CMD_DISPLAY_ALL_ON_RES,	/* set entire display on */
		SSD1306_CMD_NORMAL,		/* set normal display mode */
		SSD1306_CMD_ADDR_MODE,                // 0x20
		  0,                                  // 0x0 Horizontal Addressing Mode
		SSD1306_CMD_DISPLAY_ON		/* switch display on */
	};

	Cpu_Delay100US(100);			/* 10ms delay */

	for (i = 0; i < sizeof(init_cmd_sequence); i++)
		ssd1306_write_cmd(init_cmd_sequence[i]);

	/* 0.1 seconds delay */
	Cpu_Delay100US(1000);

	ssd1306_clear();

}

void ssd1306_set_address(uint8_t x, uint8_t y)
{
	ssd1306_write_cmd(SSD1306_CMD_SET_COLUMN_ADDR);
	ssd1306_write_cmd(x);
	ssd1306_write_cmd(SSD1306_LCD_WIDTH - 1);
	ssd1306_write_cmd(SSD1306_CMD_SET_PAGE_ADDR);
	ssd1306_write_cmd(y);
	ssd1306_write_cmd((SSD1306_LCD_HEIGHT / 8) - 1);

}

uint8_t ssd1306_set_cursor(uint8_t row, uint8_t column)
{
	if (row > (SSD1306_LCD_HEIGHT / 8) - 1) return 1;
	if (column > (SSD1306_LCD_WIDTH / 6) - 1) return 1;
	ssd1306_set_address(column * 6, row);

	return 0;
}

void ssd1306_clear(void)
{
	uint8_t row, column, buffer[8] = {0, 0, 0, 0, 0, 0, 0, 0};

	ssd1306_write_cmd(SSD1306_CMD_DISPLAY_OFF);

	for (row = 0; row < SSD1306_LCD_HEIGHT / 8; row++) {
		ssd1306_set_address(0, row);
		for (column = 0; column < SSD1306_LCD_WIDTH; column += 1) {
			/* Clear the display - 8x8 pixels at the time */
			ssd1306_write_data(buffer, 8);
		}
	}

	ssd1306_write_cmd(SSD1306_CMD_DISPLAY_ON);

}

void ssd1306_putc(uint8_t c)
{
	if (c < 32 || c > 127) c = ' ';

	ssd1306_write_data(&ssd1306_font[((c - 32) * 6)], 6);

}

void ssd1306_puts(char string[])
{
	uint8_t i = 0;
	while (string[i] != '\0') {
		ssd1306_putc(string[i]);
		i++;
	}

}
void ssd1306_BIGputc(uint8_t y, uint8_t x, uint8_t c)
{
	//uint8_t ch, i = 0;
	//ch = c -32;
	if (c < 32 || c > 127) c = ' ';
	
	ssd1306_set_address(x,y);//ssd1306_set_cursor(y,x);
	ssd1306_write_data(&ssd1306_BIGfont[((c - 32) * 16)], 8);
	ssd1306_set_address(x,y+1);//ssd1306_set_cursor(y+1,x);
	ssd1306_write_data(&ssd1306_BIGfont[((c - 32) * 16)+8], 8);
}

void ssd1306_BIGputs(uint8_t y, uint8_t x, char string[])
{
	uint8_t i = 0;
	
	while (string[i] != '\0') {
		if (x > 120) { 
				x = 0; 
				y++; }
		ssd1306_BIGputc(y, x, string[i]);
		x = x + 8;
		i++;
	}

}
