/*
 * AM2315.c
 *
 *  Created on: 28 juil. 2020
 *      Author: Dominique
 */
#include "stm32l0xx_hal.h"
#include "AM2315.h"
#include <stdio.h>

I2C_HandleTypeDef hi2c1;

/**************************************************************************/
/*!
    @brief  Helper to read the temperature & humidity from the device
    @return True if successful data read, note you can only read once every 2
   seconds!
*/
/**************************************************************************/
uint8_t AM2315_readData(void) {
  uint8_t reply[10];
/*
  if (lastreading) {
    if (millis() > lastreading) {
      if ((millis() - lastreading) <
          2000) {     // has it been less than 2 seconds since?
        return false; // bail, they need to wait longer!
      }
    } else {
      // millis() is less than the last reading, so we wrapped around!
      lastreading = millis();
      return false; // bail again
    }
  }
  lastreading = millis(); // reset our timer
*/
  // Wake up the sensor
   // write single byte twice to wake up
   // single write is not enough
  reply[0] = 0x00;
  HAL_I2C_Master_Transmit(&hi2c1, AM2315_I2CADDR, reply, 1, 0x10);
  HAL_I2C_Master_Transmit(&hi2c1, AM2315_I2CADDR, reply, 1, 0x10);

  // OK lets ready!
  reply[0] = AM2315_READREG;
  reply[1] = 0x00;  // read from adr 0x00
  reply[2] = 0x04;  // read 4 bytes
  HAL_I2C_Master_Transmit(&hi2c1, AM2315_I2CADDR, reply, 3, 0x10);

    // wait 2ms before we start to read reg
  HAL_Delay(2) ;

  HAL_I2C_Master_Receive(&hi2c1, AM2315_I2CADDR, reply, 8, 0x100);

  if (reply[0] != AM2315_READREG)
    return false;
  if (reply[1] != 4)
    return false; // bytes req'd

  humidity = reply[2];
  humidity *= 256;
  humidity += reply[3];
  humidity /= 10;

  temp = reply[4] & 0x7F;
  temp *= 256;
  temp += reply[5];
  temp /= 10;

  // change sign
  if (reply[4] >> 7)
    temp = -temp;

  return true;
}

/**************************************************************************/
/*!
    @brief  Read and return the temperature, note you can only read once every 2
   seconds
    @return Floating point Celsius temperature on success, 0 on failure
*/
/**************************************************************************/
float AM2315_readTemperature(void) {
  if (!AM2315_readData())
    return 0;
  return temp;
}

/**************************************************************************/
/*!
    @brief  Read and return the humidity, note you can only read once every 2
   seconds
    @return Floating point percentace humidity on success, 0 on failure
*/
/**************************************************************************/
float AM2315_readHumidity(void) {
  if (!AM2315_readData())
    return 0;
  return humidity;
}

/**************************************************************************/
/*!
  @brief This method returns both temperature and humidity in a single call and
  using a single I2C request. If you want to obtain both temperature and
  humidity when you sample the sensor, be aware that calling readTemperature()
  and readHumidity() in rapid succession may swamp the sensor and result in
  invalid readingings (the AM2315 manual advisess that continuous samples must
  be at least 2 seconds apart). Calling this method avoids the double I2C
  request.
  @param t Pointer to float to store temperature data in
  @param h Pointer to float to store humidity data in
  @return True if successful data read, note you can only read once every 2
  seconds
 */
uint8_t AM2315_readTemperatureAndHumidity(float *t, float *h) {
  if (!AM2315_readData())
    return false;

  *t = temp;
  *h = humidity;

  return true;
}
