/*
 * GPS.c
 *
 *  Created on: 14 oct. 2019
 *      Author: Dominique
 */

#include "GPS.h"
#include "stm32l0xx_hal.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

extern UART_HandleTypeDef huart5;

#define	_GPS_USART					huart5
#define	_GPS_DEBUG					0

GPS_t GPS;
//##################################################################################################################
double convertDegMinToDecDeg (float degMin)
{
  double min = 0.0;
  double decDeg = 0.0;

  //get the minutes, fmod() requires double
  min = fmod((double)degMin, 100.0);

  //rebuild coordinates in decimal degrees
  degMin = (int) ( degMin / 100 );
  decDeg = degMin + ( min / 60 );

  return decDeg;
}
//##################################################################################################################
void	GPS_Init(void)
{
	GPS.rxIndex=0;
	HAL_UART_Receive_IT(&_GPS_USART,&GPS.rxTmp,1);
}
//##################################################################################################################
void	GPS_CallBack(void)
{
	GPS.LastTime=HAL_GetTick();
	if(GPS.rxIndex < sizeof(GPS.rxBuffer)-2)
	{
		GPS.rxBuffer[GPS.rxIndex] = GPS.rxTmp;
		GPS.rxIndex++;
	}
	HAL_UART_Receive_IT(&_GPS_USART,&GPS.rxTmp,1);
}
//##################################################################################################################
void	GPS_Process(void)
{
	if( (HAL_GetTick()-GPS.LastTime>50) && (GPS.rxIndex>0))
	{
		char	*str;
		#if (_GPS_DEBUG==1)
		printf("%s",GPS.rxBuffer);
		#endif
		str=strstr((char*)GPS.rxBuffer,"$GPGGA,");
		if(str!=NULL)
		{
			memset(&GPS.GPGGA,0,sizeof(GPS.GPGGA));
			//sscanf(str,"$GPGGA,%2hhd%2hhd%2hhd.%3hd,%f,%c,%f,%c,%hhd,%hhd,%f,%f,%c,%hd,%s,*%2s\r\n",&GPS.GPGGA.UTC_Hour,&GPS.GPGGA.UTC_Min,&GPS.GPGGA.UTC_Sec,&GPS.GPGGA.UTC_MicroSec,&GPS.GPGGA.Latitude,&GPS.GPGGA.NS_Indicator,&GPS.GPGGA.Longitude,&GPS.GPGGA.EW_Indicator,&GPS.GPGGA.PositionFixIndicator,&GPS.GPGGA.SatellitesUsed,&GPS.GPGGA.HDOP,&GPS.GPGGA.MSL_Altitude,&GPS.GPGGA.MSL_Units,&GPS.GPGGA.AgeofDiffCorr,GPS.GPGGA.DiffRefStationID,GPS.GPGGA.CheckSum);
			//sscanf(str,"$GPGGA,%f,%f,%c,%f,%c,%hhd,%hhd,%f,%f,%c,%hd,%s",&GPS.GPGGA.UTC_Time,&GPS.GPGGA.Latitude,&GPS.GPGGA.NS_Indicator,&GPS.GPGGA.Longitude,&GPS.GPGGA.EW_Indicator,&GPS.GPGGA.PositionFixIndicator,&GPS.GPGGA.SatellitesUsed,&GPS.GPGGA.HDOP,&GPS.GPGGA.MSL_Altitude,&GPS.GPGGA.MSL_Units,&GPS.GPGGA.AgeofDiffCorr,GPS.GPGGA.DiffRefStationID);

			str = strchr(str, ',')+1;
			//parseTime(str);
			GPS.GPGGA.UTC_Time = atof(str);
			// parse out latitude
			str = strchr(str, ',')+1;
			//parseLat(str);
			GPS.GPGGA.Latitude = atof(str);
			str = strchr(str, ',')+1;
			//if(!parseLatDir(p)) return false;
			GPS.GPGGA.NS_Indicator = *str;
			// parse out longitude
			str = strchr(str, ',')+1;
			//parseLon(str);
			GPS.GPGGA.Longitude = atof(str);
			str = strchr(str, ',')+1;
			//if(!parseLonDir(p)) return false;
			GPS.GPGGA.EW_Indicator = *str;
			str = strchr(str, ',')+1;
			if (*str != ',')
			    {
			      GPS.GPGGA.PositionFixIndicator = atoi(str);
			    }

			str = strchr(str, ',')+1;
			if (*str != ',')
			    {
			      GPS.GPGGA.SatellitesUsed = atoi(str);
			    }
			str = strchr(str, ',')+1;
			if (*str != ',')
			    {
			      GPS.GPGGA.HDOP = atof(str);
			    }
			str = strchr(str, ',')+1;
			if (*str != ',')
			    {
			      GPS.GPGGA.MSL_Altitude = atof(str);
			    }

			 str = strchr(str, ',')+1;
			 str = strchr(str, ',')+1;
			 if (*str != ',')
			    {
			      GPS.GPGGA.Geoid_Separation = atof(str);
			    }

			if(GPS.GPGGA.NS_Indicator==0) GPS.GPGGA.NS_Indicator='-';
			if(GPS.GPGGA.EW_Indicator==0) GPS.GPGGA.EW_Indicator='-';
			GPS.GPGGA.UTC_Hour = (uint32_t)GPS.GPGGA.UTC_Time / 10000;
			GPS.GPGGA.UTC_Min = ((uint32_t)GPS.GPGGA.UTC_Time % 10000)/100;
			GPS.GPGGA.UTC_Sec = (uint32_t)GPS.GPGGA.UTC_Time % 100;
			GPS.GPGGA.LatitudeDecimal=convertDegMinToDecDeg(GPS.GPGGA.Latitude);
			GPS.GPGGA.LongitudeDecimal=convertDegMinToDecDeg(GPS.GPGGA.Longitude);
			str=strstr((char*)GPS.rxBuffer,"$GPRMC,");
			if(str!=NULL)
				{
					str = strchr(str, ',')+1;
					str = strchr(str, ',')+1;
					str = strchr(str, ',')+1;
					str = strchr(str, ',')+1;
					str = strchr(str, ',')+1;
					str = strchr(str, ',')+1;

					str = strchr(str, ',')+1;

					str = strchr(str, ',')+1;

					str = strchr(str, ',')+1;
					if (*str != ',')
						    {
						      GPS.GPGGA.gps_date = atoi(str);
						    }

					GPS.GPGGA.gps_day = GPS.GPGGA.gps_date/10000;
					GPS.GPGGA.gps_month = (GPS.GPGGA.gps_date % 10000)/100;
					GPS.GPGGA.gps_year = GPS.GPGGA.gps_date % 100;

				}
		}
		memset(GPS.rxBuffer,0,sizeof(GPS.rxBuffer));
		GPS.rxIndex=0;
	}
	HAL_UART_Receive_IT(&_GPS_USART,&GPS.rxTmp,1);

}
//##################################################################################################################


