#include "stm32l0xx_hal.h"
#include "VEML6070.h"
#include <stdio.h>

I2C_HandleTypeDef hi2c1;

void initialize_VEML6070(void)
{
	// Read ARA to clear interrupt
	cmd = VEML6070_CMD_DEFAULT;
	uint8_t address;
	VEML6070_read_byte(VEML6070_ADDR_ARA, &address);
	// Initialize command register
	VEML6070_write_byte(VEML6070_ADDR_CMD, &cmd);
	HAL_Delay(300);
}
void enable_sensor(void)
{
	cmd &= ~VEML6070_CMD_SD;
	VEML6070_write_byte(VEML6070_ADDR_CMD, &cmd);
}
void disable_sensor(void)
{
	cmd |= VEML6070_CMD_SD;
	VEML6070_write_byte(VEML6070_ADDR_CMD, &cmd);
}
uint16_t read_uvs_step(void)
{
	uint8_t lsb, msb;
	uint16_t data;
	VEML6070_read_byte(VEML6070_ADDR_DATA_MSB, &msb);
	VEML6070_read_byte(VEML6070_ADDR_DATA_LSB, &lsb);
	data = ((uint16_t)msb << 8) | (uint16_t)lsb;
	return data;
}
RISK_LEVEL convert_to_risk_level(uint16_t uvs_step)
{
	uint16_t risk_level_mapping_table[4] = {1121, 2241, 2988, 4109};
	uint16_t i;
	for (i = 0; i < 4; i++)
	{
		if (uvs_step <= risk_level_mapping_table[i])
		{
			break;
		}
	}
	return (RISK_LEVEL)i;
}
int VEML6070_read_byte(uint16_t addr, uint8_t *data)
{
	int err = 0;
	int retry = 3;
	// Read byte data
	while (retry--)
	{
		err = HAL_I2C_Master_Receive(&hi2c1, addr, data, 1, 0x100);
		if (err >= 0)
		return err;
	}
	return err;
}
int VEML6070_write_byte(uint16_t addr, uint8_t *data)
{
	int err = 0;
	int retry = 3;
	while (retry--)
	{
		// Send slave address & command
		err = HAL_I2C_Master_Transmit(&hi2c1, addr, data, 1, 0x100);
		if (err >= 0)
	return 0;}
	return err;
}
