/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "MPL3115A2.h"
#include "TSL2591_v2.h"
#include "VEML6070.h"
#include "AM2315.h"
#include "GPS.h"
#include <stdio.h>
#include <string.h>
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
typedef struct{

	float temperature1;
	float humidity;
	float pressure;
	uint8_t wind_dir;
	float wind_speed;
	uint16_t UV;
	uint16_t LUX;
}mesurescapteurs;

/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
//#define adr_T9602 0x50  //0x50 in 8bits mode or 0x28 in 7bits mode
#define MPLAddressI2C 0xC0;  //This is the slave address of the device

#define false 0
#define true 1
/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc;

I2C_HandleTypeDef hi2c1;

RTC_HandleTypeDef hrtc;

TIM_HandleTypeDef htim21;

UART_HandleTypeDef huart2;
UART_HandleTypeDef huart5;

/* USER CODE BEGIN PV */
mesurescapteurs captmes;
uint32_t TimeStamp, lastWindCheck;
uint16_t seconds = 0, nextsec = 2, nmes;

const uint32_t tab_dirvent[16][2] = {
    {0, 3051}, {1, 2259}, {2, 2412}, {3, 814},
    {4, 876}, {5, 687}, {6, 1406}, {7, 1086},
    {8, 1858}, {9, 1684}, {10, 2789}, {11, 2728},
    {12, 3262}, {13, 3104}, {14, 3192}, {15, 2914}
};
uint16_t valCAN, compt_anemo;
char msg[100]={0}, validSend, first;
RTC_TimeTypeDef time;
RTC_DateTypeDef date;
float windspeedsum, tempersum, pressuresum, humitysum, UVsum, LUXsum;
uint8_t sigfoxdata[14];
/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_ADC_Init(void);
static void MX_I2C1_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_USART5_UART_Init(void);
static void MX_RTC_Init(void);
static void MX_TIM21_Init(void);
/* USER CODE BEGIN PFP */
//****************************************************
//		Wind direction calculation
//****************************************************
uint8_t dir_vent(uint32_t Vin);
//***************************************
//Returns the instataneous wind speed
//**************************************
float get_wind_speed(void);
//******************************************
// Measures proceed
//******************************************
void mesures(mesurescapteurs *mescapt);
//****************************************************
//   Wind_Speed Interruption
//***************************************************
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin);
/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */

  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_ADC_Init();
  MX_I2C1_Init();
  MX_USART2_UART_Init();
  MX_USART5_UART_Init();
  MX_RTC_Init();
  MX_TIM21_Init();
  /* USER CODE BEGIN 2 */
  HAL_GPIO_WritePin(EN3V3_GPIO_Port, EN3V3_Pin, GPIO_PIN_SET);// To power the sensors
  HAL_Delay(100);// Wait for supply stabilization
  HAL_ADC_Start(&hadc);
  GPS_Init();
    do{
    	  GPS_Process();
    	  HAL_Delay(100);
    	  //HAL_IWDG_Refresh(&hiwdg);
      }while(GPS.GPGGA.PositionFixIndicator !=1);//while(GPS.GPGGA.status !='A');
    time.Hours = GPS.GPGGA.UTC_Hour ;
    time.Minutes = GPS.GPGGA.UTC_Min;
    time.Seconds = GPS.GPGGA.UTC_Sec;
    date.Year = GPS.GPGGA.gps_year; 	//date.Year = 20;
    date.Month = GPS.GPGGA.gps_month;	//date.Month = 2;
    date.Date = GPS.GPGGA.gps_day;	// date.Date = 19;
    HAL_RTC_SetTime(&hrtc, &time, RTC_FORMAT_BIN); //mettre la RTC à l’heure "time"
    HAL_RTC_SetDate(&hrtc, &date, RTC_FORMAT_BIN);
    __HAL_UART_DISABLE_IT(&huart5, UART_IT_RXNE);
    HAL_UART_DeInit(&huart5);
    HAL_NVIC_DisableIRQ(EXTI4_15_IRQn);
    HAL_UART_DeInit(&huart2);
    //HAL_GPIO_WritePin(EN3V3_GPIO_Port, EN3V3_Pin, GPIO_PIN_RESET);// Power the sensors off
    compt_anemo = 0; windspeedsum = 0; tempersum = 0;  pressuresum = 0; humitysum = 0; UVsum = 0; LUXsum = 0;
    nmes = 0; validSend = 0; first = 1;

  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
	  //HAL_SuspendTick();
	  //HAL_NVIC_DisableIRQ(7);
	  //HAL_PWR_EnterSTOPMode(PWR_LOWPOWERREGULATOR_ON, PWR_STOPENTRY_WFE);
	  //HAL_PWR_EnterSTANDBYMode();
	  //HAL_ResumeTick();
	  //HAL_NVIC_EnableIRQ(7);

	  //HAL_IWDG_Refresh(&hiwdg);
	  if(validSend ){
		  if (first){
			  //HAL_GPIO_WritePin(EN3V3_GPIO_Port, EN3V3_Pin, GPIO_PIN_SET);// To power the sensors
			  HAL_Delay(300);// Wait for supply stabilization
			  //HAL_IWDG_Refresh(&hiwdg);
			  MPL_Init();
			  TSL2591_Init();
			  initialize_VEML6070();
			  HAL_TIM_Base_Start_IT(&htim21);
			  HAL_UART_MspInit(&huart2);
			  MX_USART2_UART_Init();//HAL_UART_Init(&huart2);
			  HAL_NVIC_EnableIRQ(EXTI4_15_IRQn);
			  first = 0;
		  }

		  //HAL_IWDG_Refresh(&hiwdg);

		  if(seconds >= nextsec){
		  	  mesures(&captmes);
		  	  tempersum = tempersum + captmes.temperature1;
		  	  pressuresum = pressuresum + captmes.pressure;
		  	  humitysum = humitysum + captmes.humidity;
		  	  windspeedsum = windspeedsum + captmes.wind_speed;
		  	  UVsum = UVsum + (float)captmes.UV;
		  	  LUXsum = LUXsum + (float)captmes.LUX;
		  	  nmes++;
		  	  nextsec = seconds + 2;
		  	  //HAL_IWDG_Refresh(&hiwdg);
		  }
		  if(nmes==5){
			  HAL_NVIC_DisableIRQ(EXTI4_15_IRQn);
			  captmes.temperature1 = tempersum/nmes;
			  captmes.pressure = pressuresum/nmes;
			  captmes.humidity = humitysum/nmes;
			  captmes.wind_speed = windspeedsum/nmes;
			  captmes.UV = (uint16_t)(UVsum/nmes);
			  captmes.LUX = (uint16_t)(LUXsum/nmes);
			  HAL_RTC_GetTime(&hrtc, &time, RTC_FORMAT_BIN); //lire heure dans RTC
			  HAL_RTC_GetDate(&hrtc, &date, RTC_FORMAT_BIN); //lire date dans RTC (non utilisée ici)
			  //HAL_IWDG_Refresh(&hiwdg);
			  /*sprintf(msg,"Date: %02d/%02d/%02d time: %02d:%02d:%02d\n",date.Date, date.Month, date.Year+2000, time.Hours, time.Minutes, time.Seconds);
			  HAL_UART_Transmit(&huart2,(uint8_t*)msg,strlen(msg),100);
			  sprintf(msg,"T=%.2f,H=%.2f,P=%.2f,Dir=%.2f,Vit=%.2f,E = %d Lux, UV = %d \n",captmes.temperature1, captmes.humidity, captmes.pressure, captmes.wind_dir*22.5, captmes.wind_speed, captmes.LUX, captmes.UV);
			  HAL_UART_Transmit(&huart2,(uint8_t*)msg,strlen(msg),100);*/

			  sigfoxdata[0] = (uint8_t)((uint16_t)(captmes.temperature1*10)>>8);
			  sigfoxdata[1] = (uint8_t)(captmes.temperature1*10);
			  sigfoxdata[2] = (uint8_t)((uint32_t)captmes.pressure>>16);
			  sigfoxdata[3] = (uint8_t)((uint32_t)captmes.pressure>>8);
			  sigfoxdata[4] = (uint8_t)captmes.pressure;
			  sigfoxdata[5] = (uint8_t)captmes.humidity;
			  sigfoxdata[6] = (uint8_t)captmes.wind_speed;
			  sigfoxdata[7] = (uint8_t)captmes.wind_dir;
			  sigfoxdata[8] = (uint8_t)(captmes.LUX >> 8);
			  sigfoxdata[9] = (uint8_t)(captmes.LUX);
			  sigfoxdata[10] = (uint8_t)(captmes.UV >> 8);
			  sigfoxdata[11] = (uint8_t)(captmes.UV);
			  HAL_UART_Transmit(&huart2,(uint8_t*)sigfoxdata, 12, 100);
			  //HAL_IWDG_Refresh(&hiwdg);
			  nextsec = seconds + 8;		//Sigfox transmission up to 8s
			  while(seconds<nextsec){
				  //HAL_IWDG_Refresh(&hiwdg);
			  }
			  HAL_UART_DeInit(&huart2);
			  //HAL_GPIO_WritePin(EN3V3_GPIO_Port, EN3V3_Pin, GPIO_PIN_RESET);// Power the sensors off
			  HAL_TIM_Base_Stop_IT(&htim21);
			  seconds = 0;
			  nextsec = 2;
			  nmes = 0;
			  windspeedsum = 0; tempersum = 0;  pressuresum = 0; humitysum = 0; UVsum = 0; LUXsum = 0;
			  validSend = 0; first = 1;
		  }

		  //HAL_Delay(2000);
	  }

	  //HAL_Delay(10000);
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Configure the main internal regulator output voltage
  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE3);
  /** Configure LSE Drive Capability
  */
  HAL_PWR_EnableBkUpAccess();
  __HAL_RCC_LSEDRIVE_CONFIG(RCC_LSEDRIVE_LOW);
  /** Initializes the CPU, AHB and APB busses clocks
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSE|RCC_OSCILLATORTYPE_MSI;
  RCC_OscInitStruct.LSEState = RCC_LSE_ON;
  RCC_OscInitStruct.MSIState = RCC_MSI_ON;
  RCC_OscInitStruct.MSICalibrationValue = 0;
  RCC_OscInitStruct.MSIClockRange = RCC_MSIRANGE_5;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_MSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART2|RCC_PERIPHCLK_I2C1
                              |RCC_PERIPHCLK_RTC;
  PeriphClkInit.Usart2ClockSelection = RCC_USART2CLKSOURCE_PCLK1;
  PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_PCLK1;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSE;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC_Init(void)
{

  /* USER CODE BEGIN ADC_Init 0 */

  /* USER CODE END ADC_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC_Init 1 */

  /* USER CODE END ADC_Init 1 */
  /** Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion)
  */
  hadc.Instance = ADC1;
  hadc.Init.OversamplingMode = DISABLE;
  hadc.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV1;
  hadc.Init.Resolution = ADC_RESOLUTION_12B;
  hadc.Init.SamplingTime = ADC_SAMPLETIME_12CYCLES_5;
  hadc.Init.ScanConvMode = ADC_SCAN_DIRECTION_FORWARD;
  hadc.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc.Init.ContinuousConvMode = ENABLE;
  hadc.Init.DiscontinuousConvMode = DISABLE;
  hadc.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc.Init.DMAContinuousRequests = DISABLE;
  hadc.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  hadc.Init.Overrun = ADC_OVR_DATA_PRESERVED;
  hadc.Init.LowPowerAutoWait = DISABLE;
  hadc.Init.LowPowerFrequencyMode = DISABLE;
  hadc.Init.LowPowerAutoPowerOff = DISABLE;
  if (HAL_ADC_Init(&hadc) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure for the selected ADC regular channel to be converted.
  */
  sConfig.Channel = ADC_CHANNEL_4;
  sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC_Init 2 */

  /* USER CODE END ADC_Init 2 */

}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.Timing = 0x00000708;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Analogue filter
  */
  if (HAL_I2CEx_ConfigAnalogFilter(&hi2c1, I2C_ANALOGFILTER_ENABLE) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Digital filter
  */
  if (HAL_I2CEx_ConfigDigitalFilter(&hi2c1, 0) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief RTC Initialization Function
  * @param None
  * @retval None
  */
static void MX_RTC_Init(void)
{

  /* USER CODE BEGIN RTC_Init 0 */

  /* USER CODE END RTC_Init 0 */

  RTC_TimeTypeDef sTime = {0};
  RTC_DateTypeDef sDate = {0};

  /* USER CODE BEGIN RTC_Init 1 */

  /* USER CODE END RTC_Init 1 */
  /** Initialize RTC Only
  */
  hrtc.Instance = RTC;
  hrtc.Init.HourFormat = RTC_HOURFORMAT_24;
  hrtc.Init.AsynchPrediv = 127;
  hrtc.Init.SynchPrediv = 255;
  hrtc.Init.OutPut = RTC_OUTPUT_DISABLE;
  hrtc.Init.OutPutRemap = RTC_OUTPUT_REMAP_NONE;
  hrtc.Init.OutPutPolarity = RTC_OUTPUT_POLARITY_HIGH;
  hrtc.Init.OutPutType = RTC_OUTPUT_TYPE_OPENDRAIN;
  if (HAL_RTC_Init(&hrtc) != HAL_OK)
  {
    Error_Handler();
  }

  /* USER CODE BEGIN Check_RTC_BKUP */

  /* USER CODE END Check_RTC_BKUP */

  /** Initialize RTC and set the Time and Date
  */
  sTime.Hours = 0;
  sTime.Minutes = 0;
  sTime.Seconds = 0;
  sTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
  sTime.StoreOperation = RTC_STOREOPERATION_RESET;
  if (HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BIN) != HAL_OK)
  {
    Error_Handler();
  }
  sDate.WeekDay = RTC_WEEKDAY_MONDAY;
  sDate.Month = RTC_MONTH_JANUARY;
  sDate.Date = 1;
  sDate.Year = 0;

  if (HAL_RTC_SetDate(&hrtc, &sDate, RTC_FORMAT_BIN) != HAL_OK)
  {
    Error_Handler();
  }
  /** Enable the WakeUp
  */
  if (HAL_RTCEx_SetWakeUpTimer_IT(&hrtc, 900, RTC_WAKEUPCLOCK_CK_SPRE_16BITS) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN RTC_Init 2 */

  /* USER CODE END RTC_Init 2 */

}

/**
  * @brief TIM21 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM21_Init(void)
{

  /* USER CODE BEGIN TIM21_Init 0 */

  /* USER CODE END TIM21_Init 0 */

  TIM_ClockConfigTypeDef sClockSourceConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_OC_InitTypeDef sConfigOC = {0};

  /* USER CODE BEGIN TIM21_Init 1 */

  /* USER CODE END TIM21_Init 1 */
  htim21.Instance = TIM21;
  htim21.Init.Prescaler = 2097;
  htim21.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim21.Init.Period = 999;
  htim21.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim21.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  if (HAL_TIM_Base_Init(&htim21) != HAL_OK)
  {
    Error_Handler();
  }
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim21, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_TIM_PWM_Init(&htim21) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim21, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sConfigOC.OCMode = TIM_OCMODE_PWM1;
  sConfigOC.Pulse = 0;
  sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
  if (HAL_TIM_PWM_ConfigChannel(&htim21, &sConfigOC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM21_Init 2 */

  /* USER CODE END TIM21_Init 2 */

}

/**
  * @brief USART2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  huart2.Instance = USART2;
  huart2.Init.BaudRate = 19200;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  huart2.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart2.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}

/**
  * @brief USART5 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART5_UART_Init(void)
{

  /* USER CODE BEGIN USART5_Init 0 */

  /* USER CODE END USART5_Init 0 */

  /* USER CODE BEGIN USART5_Init 1 */

  /* USER CODE END USART5_Init 1 */
  huart5.Instance = USART5;
  huart5.Init.BaudRate = 9600;
  huart5.Init.WordLength = UART_WORDLENGTH_8B;
  huart5.Init.StopBits = UART_STOPBITS_1;
  huart5.Init.Parity = UART_PARITY_NONE;
  huart5.Init.Mode = UART_MODE_TX_RX;
  huart5.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart5.Init.OverSampling = UART_OVERSAMPLING_16;
  huart5.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart5.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart5) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART5_Init 2 */

  /* USER CODE END USART5_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, EN_GPS_Pin|EN3V3_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(Sleep_Fox_GPIO_Port, Sleep_Fox_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pins : PA0 PA6 PA7 PA8
                           PA9 PA10 PA11 PA12 */
  GPIO_InitStruct.Pin = GPIO_PIN_0|GPIO_PIN_6|GPIO_PIN_7|GPIO_PIN_8
                          |GPIO_PIN_9|GPIO_PIN_10|GPIO_PIN_11|GPIO_PIN_12;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pins : EN_GPS_Pin EN3V3_Pin */
  GPIO_InitStruct.Pin = EN_GPS_Pin|EN3V3_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : Wind_Speed_Pin */
  GPIO_InitStruct.Pin = Wind_Speed_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_FALLING;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(Wind_Speed_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : Sleep_Fox_Pin */
  GPIO_InitStruct.Pin = Sleep_Fox_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(Sleep_Fox_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pins : PB1 PB5 */
  GPIO_InitStruct.Pin = GPIO_PIN_1|GPIO_PIN_5;
  GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI4_15_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI4_15_IRQn);

}

/* USER CODE BEGIN 4 */
//****************************************************
//		Wind direction calculation
//****************************************************
uint8_t dir_vent(uint32_t Vin)
{
    uint8_t i;
    uint32_t wd;
    for (i = 0; i < 16; i ++) {
        if (Vin > tab_dirvent[i][1] * 0.98 && Vin < tab_dirvent[i][1] * 1.02) {
            wd = tab_dirvent[i][0];
            return((uint8_t)wd);
        }
    }
    return -1;
}
uint32_t calcVin (void)
{
	uint32_t VIN;
    if(HAL_ADC_PollForConversion(&hadc, 100000) == HAL_OK){
    		  VIN = HAL_ADC_GetValue(&hadc);
    		  VIN = VIN*3300/4096;
    		  //rint = rint*3300/(3300-rint);
    }
    return(VIN);
}
//***************************************
//Returns the instataneous wind speed
//**************************************
float get_wind_speed(void)
{
  float deltaTime = HAL_GetTick() - lastWindCheck;

  deltaTime /= 1000.0; //Convert to seconds

  float windSpeed = (float)compt_anemo / deltaTime; //3 / 0.750s = 4

  compt_anemo = 0; //Reset and start watching for new wind
  lastWindCheck = HAL_GetTick();

  windSpeed = 2.4 * windSpeed; // 1 pulse per seconds --> 2,4 km/h

  return(windSpeed);
}
//******************************************
// Measures proceed
//******************************************
void mesures(mesurescapteurs *mescapt)
{
    //uint8_t wake_cmd[1];
    //uint8_t data[4];
	float humi, temp;
    //MPL_Read_MPL_Data();
    HAL_Delay(100);
//****  Digital Humidity Sensor T9602 *****
   /* data[0] = 0x00;
    data[1] = 0x00;
    data[2] = 0x00;
    data[3] = 0x00;
    wake_cmd[0] = 0x00;
    HAL_I2C_Master_Transmit(&hi2c1, adr_T9602, wake_cmd, 1, 1000);//i2c.write(adr_T9602, wake_cmd, 1);
    HAL_Delay(50);
    HAL_I2C_Master_Receive(&hi2c1, adr_T9602, data, 4, 1000);//i2c.read(adr_T9602, data, 4);

    mescapt->humidity = ((((uint16_t)data[0] & 63) << 8) + (uint16_t)data[1]) / 163.84;
    mescapt->temperature1 = (((data[2] << 6) + (data[3] / 4)) / 99.29) - 40;
    */
//****  Digital Humidity Sensor AM2315 *****
    AM2315_readTemperatureAndHumidity(&temp, &humi);
    mescapt->temperature1 = temp;
    mescapt->humidity =humi;
    mescapt->pressure = MPL_Pressure();
    mescapt->wind_dir = dir_vent(calcVin());
    mescapt->wind_speed = get_wind_speed();
    mescapt->LUX = TSL2591_Read_Lux();
    mescapt->UV = read_uvs_step();
}
//****************************************************
//   Wind_Speed Interruption
//***************************************************
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin){
	if (GPIO_Pin == GPIO_PIN_5)
	    {
			compt_anemo++;
	    }
}
//****************************************************
//   UART Interruptions
//***************************************************
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart){

	/*if (huart->Instance == USART2)  //UART for serial Sigfox
	    {

			if (strcmp(datain, "dem1")==0){
				askmes = 1;
			 	datain[0] = 0;
			 	HAL_UART_Receive_IT(&huart2,(uint8_t*)datain, 4 );
			  }
			else {
				askmes = 0;
				datain[0] = 0;
				HAL_UART_Receive_IT(&huart2,(uint8_t*)datain, 4 );
			}
	    }*/
	if (huart->Instance == USART5)  //UART for GPS
		{
			GPS_CallBack();
		}

}
/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
