#include <Arduino.h>
#include <Wire.h>
#include <SPI.h>
#include <Adafruit_BMP3XX.h>
#include <T9602.h>
#include "PMS.h"
#include "WindSensor.h"
#include <WiFi.h>
#include <NTPClient.h>
#include <WiFiUdp.h>
#include <driver/rtc_io.h>

//#define sigfox

#define uS_TO_S_FACTOR 1000000  /* Conversion factor for micro seconds to seconds */
#define SLEEP_TIME    (12*60)  //Sleep time = n*60*1000000 us for n in minutes
#define SEALEVELPRESSURE_HPA (1013.25)
//#define EN5V    0
gpio_num_t EN5V = GPIO_NUM_0;
#define com3v3  15       //IO15 used to connect 3v3
#define Wspeed  27       //Wind Spped Pin IO27
#define Wdir    34      //Analog Wind Direction Pin
#define Ubat    A0      //Analog battery voltage Pin
#define cs_Fox  04      //CS pin of the Sigfox module
#define raz_Fox 14      //RAZ pin of the Sigfox modulex
#define ack_Fox 16      //ACK pin of the Sigfox module
#define SERIAL1_RXPIN 13
#define SERIAL1_TXPIN 5
//senseBox ID
#define SENSEBOX_ID "5ef78b23ee9b25001b1b76d0"

// *******   Sensor IDs   **********************
// Temperature
#define SENSOR1_ID "5ef78b23ee9b25001b1b76d6"
// Humidity
#define SENSOR2_ID "5ef78b23ee9b25001b1b76d5"
// Pressure
#define SENSOR3_ID "5ef78b23ee9b25001b1b76d4"
// Windspeed
#define SENSOR4_ID "5ef78b23ee9b25001b1b76d3"
// PM10
#define SENSOR5_ID "5ef78b23ee9b25001b1b76d2"
// Battery Voltage
#define SENSOR6_ID "5ef78b23ee9b25001b1b76d1"
// **************************************************
void sendWakeUp();
void sendToSleep();
void getDeviceInfo();
void setRCZ1();
void sendBit();
void sendPayload(uint8_t * payload, uint8_t payloadLength, uint8_t downlink);
void recvDataSPI();
void sendDataSPI(uint8_t bufLength);
void printByte(uint8_t b);
// *********  Procedure for connecting to the opensensemap server ***** 
// ********* and transmitting measurement data  *******
void postFloatValue (float measurement, int aftercomma, String sensorId);
void waitForServerResponse (void);

PMS pms(Serial1);   //PMS5003 is connected on pin GPIO9 (RXD1) and pin GPIO10 (TXD1)
PMS::DATA dataPMS;
Adafruit_BMP3XX bmp388; // I2C388;
T9602 humsens;  
WindSensor ws(Wspeed, Wdir);
WiFiUDP ntpUDP;
// You can specify the time server pool and the offset (in seconds, can be
// changed later with setTimeOffset() ). Additionaly you can specify the
// update interval (in milliseconds, can be changed using setUpdateInterval() ).
NTPClient timeClient(ntpUDP, "europe.pool.ntp.org", 0, 60000);
// Use WiFiClient class to create TCP connections
WiFiClient client;
const SPISettings spiConf = SPISettings(125000, MSBFIRST, SPI_MODE1);
const char *SSID = "Livebox-6CC6";  // "BTS_SN_E104"
const char *mdp = "36536FC23446D2C623F5927AC3" ; // "jepensedoncjesuis"
//const char* host = "saint-cricq.com";
char server[] = "ingress.opensensemap.org";
const int httpPort = 82;
//Measurement parameters
const unsigned int postingInterval = 300000; //Uploadintervall in Milliseconds
// SPI buffers for rx and tx
uint8_t rxBuffer[40];
uint8_t txBuffer[40];
uint8_t errorCode = 255;
uint8_t ol2385State = 255;

uint8_t sigfoxdata[12] = {0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x53, 0x69, 0x67, 0x66, 0x6f, 0x78}; // "Hello sigfox"
float temperature, humidity, WinSpeed,pressure, Vbat;
float sumtemp, sumpres, sumhumi, sumWindSpeed;
int i;
uint16_t VbatFox;
uint32_t unixtime, pascals;
RTC_DATA_ATTR int bootCount = 0;
long lastSecond; //The millis counter to see when a second rolls by
volatile long lastWindIRQ = 0;
volatile byte windClicks = 0;
long lastWindCheck = 0;
int windDir = 0;
float windDirAvg = 0;
bool connected = false;
/*
IPAddress ip(192, 168, 1, 32);
IPAddress dns(192, 168, 1, 1);
IPAddress gateway(192, 168, 1, 1);
IPAddress subnet(255, 255, 255, 0);
*/
//************
//Method to print the reason by which ESP32
//has been awaken from sleep
//************
void print_wakeup_reason(){
  esp_sleep_wakeup_cause_t wakeup_reason;

  wakeup_reason = esp_sleep_get_wakeup_cause();

  switch(wakeup_reason)
  {
    case ESP_SLEEP_WAKEUP_EXT0 : Serial.println("Wakeup caused by external signal using RTC_IO"); break;
    case ESP_SLEEP_WAKEUP_EXT1 : Serial.println("Wakeup caused by external signal using RTC_CNTL"); break;
    case ESP_SLEEP_WAKEUP_TIMER : Serial.println("Wakeup caused by timer"); break;
    case ESP_SLEEP_WAKEUP_TOUCHPAD : Serial.println("Wakeup caused by touchpad"); break;
    case ESP_SLEEP_WAKEUP_ULP : Serial.println("Wakeup caused by ULP program"); break;
    default : Serial.printf("Wakeup was not caused by deep sleep: %d\n",wakeup_reason); break;
  }
}
//************************** 
// interrupt function for the anamometer
// Activated by the magnet in the anemometer (2 ticks per rotation), attached to input Wspeed  IO32
//**************************
void wspeedIRQ()

{
	if (millis() - lastWindIRQ > 10) // Ignore switch-bounce glitches less than 10ms (142MPH max reading) after the reed switch closes
	{
		lastWindIRQ = millis(); //Grab the current time
		windClicks++; //There is 1.492MPH for each click per second.
	}
}
//***************
//Returns the instataneous wind speed
//***************
float get_wind_speed()
{
	float deltaTime = millis() - lastWindCheck; 

	deltaTime /= 1000.0; //Convert to seconds

	float windSpeed = (float)windClicks / deltaTime; 

	windClicks = 0; //Reset and start watching for new wind
	lastWindCheck = millis();

	windSpeed = 2.4 * windSpeed; // 1 pulse per seconds --> 2,4 km/h

	return(windSpeed);
}


void setup() {
  digitalWrite(com3v3, LOW);
  pinMode(com3v3, OUTPUT);
  //digitalWrite(EN5V, LOW);
  //pinMode(EN5V, OUTPUT);
  rtc_gpio_hold_dis(EN5V);
  rtc_gpio_init(EN5V);
  rtc_gpio_set_direction(EN5V,RTC_GPIO_MODE_OUTPUT_ONLY);
  rtc_gpio_set_level(EN5V,0); //GPIO LOW
  // input from wind meters windspeed sensor
  ws.initWindSensor();
  // attach external interrupt pins to IRQ functions
	attachInterrupt(Wspeed, wspeedIRQ, FALLING);
// initalize the ack and chip select pins:
  pinMode(ack_Fox, INPUT);
  pinMode(cs_Fox, OUTPUT);
  //pinMode(raz_Fox, OUTPUT);
  digitalWrite(cs_Fox, HIGH);
  //digitalWrite(EN5V, HIGH);
  rtc_gpio_set_level(EN5V,1);   //Enable the 5V supply for the PMS
  Serial.begin(9600);
  delay(1000);
  Serial1.begin(9600,SERIAL_8N1, SERIAL1_RXPIN, SERIAL1_TXPIN);//Serial1.begin(9600);  // 
  digitalWrite(com3v3, HIGH);
  
  Serial.println("\n");
     // start the SPI library:
  SPI.begin(18, 19, 23, 33);
  // set SPI settings expected by module
  SPI.beginTransaction(spiConf);
  //digitalWrite(raz_Fox, HIGH);
  delay(300);
  Serial.println("Starting...");
  //sendWakeUp();
  //getDeviceInfo();
  if (!bmp388.begin_I2C()) {
    Serial.println("Could not find a valid BMP3 sensor, check wiring!");
    while (1);
  }
  humsens.begin();
  // Set up oversampling and filter initialization
  bmp388.setTemperatureOversampling(BMP3_OVERSAMPLING_8X);
  bmp388.setPressureOversampling(BMP3_OVERSAMPLING_4X);
  bmp388.setIIRFilterCoeff(BMP3_IIR_FILTER_COEFF_3);
  //bmp.setOutputDataRate(BMP3_ODR_50_HZ);
  pms.wakeUp();
  connected = false;
   // turn on interrupts
	interrupts();
   //++bootCount;
  //Serial.println("Boot number: " + String(bootCount));

  //Print the wakeup reason for ESP32
  //print_wakeup_reason();

  /*
  First we configure the wake up source
  We set our ESP32 to wake up every 6 minutes
  */
  /*esp_sleep_enable_timer_wakeup(SLEEP_TIME * uS_TO_S_FACTOR);
  Serial.println("Setup ESP32 to sleep for every " + String(SLEEP_TIME) +
  " Seconds");
*/
  WiFi.mode(WIFI_STA);
  //WiFi.config(ip, dns, gateway, subnet);
  //WiFi.persistent(false);
  WiFi.begin(SSID,mdp);
  
  Serial.print("Connexion en cours ");
  /*while(WiFi.status()!=WL_CONNECTED)
  {
    Serial.print(".");
    delay(100);
  }*/
  for (int timeout = 0; timeout < 20; timeout++) { //max 20 seconds
    int status = WiFi.status();
    if ((status == WL_CONNECTED) || (status == WL_CONNECT_FAILED) || (status == WL_NO_SSID_AVAIL))
      break;
    Serial.print(".");
    delay(1000);
  }
   if (WiFi.status() != WL_CONNECTED) {
    Serial.print("Failed to connect to " );
    Serial.println(SSID);
    
    WiFi.disconnect();
    esp_restart();
  }
  else{
    Serial.println("\n");
    Serial.println("Login established");
    Serial.print("Assigned IP address");
    Serial.println(WiFi.localIP());
    timeClient.begin();
    connected = true;
  }
  
  
  //**************************************************************
  // ancienne boucle "loop" devenue inutile en mode sommeil profond
  // car déclenchement d'un reset au moment du reveil
  //**************************************************************
  
  pms.passiveMode();    // Switch to passive mode for the Particles sensor
  sumpres = 0; sumtemp = 0 ; sumhumi = 0; sumWindSpeed = 0;
  if (! bmp388.performReading()) {
    Serial.println("Failed to perform reading :(");
    return;
    }
  delay(1000);
  pressure = bmp388.pressure;
  for(i = 0; i<10; i++){
    if (! bmp388.performReading()) {
    Serial.println("Failed to perform reading :(");
    return;
    }
     // Temperature measurement
     //temperature = bmp388.temperature;
     temperature = humsens.GetTemperature();
    
  // Humidity measurement
    humidity = humsens.GetHumidity();
    float currentSpeed = get_wind_speed();
    ws.determineWindDir();
    windDir = ws.getWindDir();
    //pressure measurement
    pressure = bmp388.pressure;
   //average of the 10 measurements 
    sumtemp = sumtemp + temperature ;
    sumpres = sumpres + pressure; 
    sumhumi = sumhumi + humidity;
    sumWindSpeed = sumWindSpeed + currentSpeed;   
    delay(1000);
  }
  temperature = sumtemp/(i);
  pressure = sumpres/i;
  pascals =(uint32_t)pressure ;
  humidity = sumhumi/i;
  WinSpeed = sumWindSpeed/i;
  windDirAvg = ws.getWindDirAvg();
  Vbat = (float)analogRead(Ubat)*1.675;// coefficient obtained by measurement
  VbatFox = (uint16_t)Vbat;
  delay(20000);   //20 sec more for stable pms data reading
  Serial.println("Send read request...");
  pms.requestRead();

  Serial.println("Wait max. 1 second for read...");
  if (pms.readUntil(dataPMS))
  {
    Serial.print("PM 1.0 (ug/m3): ");
    Serial.println(dataPMS.PM_AE_UG_1_0);

    Serial.print("PM 2.5 (ug/m3): ");
    Serial.println(dataPMS.PM_AE_UG_2_5);

    Serial.print("PM 10.0 (ug/m3): ");
    Serial.println(dataPMS.PM_AE_UG_10_0);
  }
  else
  {
    Serial.println("No data.");
  }

  pms.sleep(); 
  Serial.print("Temp : ");
  Serial.print(temperature);
  Serial.println(" ºC");
  Serial.print("Pres : ");
  Serial.print(pascals); Serial.println(" Pa");
  Serial.print("Hum : ");
  Serial.print(humidity); Serial.println(" %");
  Serial.print("Approx. Altitude = ");
  Serial.print(bmp388.readAltitude(SEALEVELPRESSURE_HPA));
  Serial.println(" m");
  Serial.print("Temp388 : ");
  Serial.print(bmp388.temperature);
  Serial.println(" ºC");
  Serial.print("Wind Speed : ");
  Serial.print(WinSpeed);
  Serial.println(" km/h");
  Serial.println();
  Serial.print("Wind Direction : ");
  Serial.print(windDirAvg);
  Serial.println(" Degres");
  Serial.println();
  timeClient.update();
  unixtime = timeClient.getEpochTime();
  Serial.print("Time : ");
  Serial.println(unixtime);
  Serial.print("Vbattery : ");
  Serial.print(Vbat);
  Serial.println(" mV");
  Serial.print("Vbatfox : ");
  Serial.print(VbatFox);
  Serial.println(" mV");
  
#ifdef sigfox  
  sigfoxdata[0] = (uint8_t)((uint16_t)(temperature*10)>>8);
	sigfoxdata[1] = (uint8_t)(temperature*10);
	sigfoxdata[2] = (uint8_t)((uint32_t)pascals>>16);
	sigfoxdata[3] = (uint8_t)((uint32_t)pascals>>8);
	sigfoxdata[4] = (uint8_t)pascals;
	sigfoxdata[5] = (uint8_t)humidity;
	//sigfoxdata[6] = (uint8_t)((uint16_t)(dataPMS.PM_AE_UG_1_0)<<8);
	sigfoxdata[6] = (uint8_t)(dataPMS.PM_AE_UG_1_0);
	//sigfoxdata[8] = (uint8_t)((uint16_t)(dataPMS.PM_AE_UG_2_5)<<8);
	sigfoxdata[7] = (uint8_t)(dataPMS.PM_AE_UG_2_5);
  sigfoxdata[8] = (uint8_t)(VbatFox>>8);
  sigfoxdata[9] = (uint8_t)(VbatFox);
	sigfoxdata[10] = (uint8_t)WinSpeed;
  sigfoxdata[11] = (uint8_t)((uint16_t)(windDirAvg/10));

	//sigfoxdata[8] = (uint8_t)windDirAvg;
  sendWakeUp();
  setRCZ1(); // apparently need to be called before each transmission
  sendPayload(sigfoxdata, 12, 0); // uncomment to send uplink message only
  sendToSleep();
 #endif
  //digitalWrite(EN5V, LOW);
  rtc_gpio_set_level(EN5V,0);
  digitalWrite(com3v3, LOW);
  /*
    // http://saint-cricq.com:82/insertMeteoLSC.php?date=1585845738&pressure=1025000&temperature=250&humidity=70&windspeed=3&winddirection=275&part3=256&part5=852&part25=458
  // Use WiFiClient class to create TCP connections
    WiFiClient client;
    if (!client.connect(host, httpPort)) {
      Serial.println("Erreur de connexion");
      return;
    }
// This will send the request to the server

 client.print(String("GET http://saint-cricq.com:82/insertMeteoLSC.php?") +
                          ("&date=") + unixtime +
                          ("&pressure=") + pascals +
                          ("&temperature=") + (uint16_t)(temperature*10) +
                          ("&humidity=") + (uint16_t)(humidity)+
                          ("&windspeed=") + (uint16_t)(WinSpeed)+
                          ("&winddirection=") + (uint16_t)(windDirAvg)+

                          " HTTP/1.1\r\n" +
                 "Host: " + host + "\r\n" +
                 "Connection: close\r\n\r\n");
    unsigned long timeout = millis();
    while (client.available() == 0) {
        if (millis() - timeout > 1000) {
            Serial.println(">>> Client Timeout !");
            client.stop();
            return;
        }
    }
    // Read all the lines of the reply from server and print them to Serial
    while(client.available()) {
        String line = client.readStringUntil('\r');
        Serial.print(line);
        
    }

    Serial.println();
    Serial.println("closing connection");
    */
   if(connected){
     postFloatValue(temperature, 1, SENSOR1_ID);
     postFloatValue(humidity, 1, SENSOR2_ID);
     postFloatValue(pressure, 1, SENSOR3_ID);
     postFloatValue(WinSpeed, 1, SENSOR4_ID);
     postFloatValue(dataPMS.PM_AE_UG_10_0 + dataPMS.PM_AE_UG_2_5, 1, SENSOR5_ID);
     postFloatValue(Vbat, 1, SENSOR6_ID);
     
   }
   
    /*
  Next we decide what all peripherals to shut down/keep on
  By default, ESP32 will automatically power down the peripherals
  not needed by the wakeup source, but if you want to be a poweruser
  this is for you. Read in detail at the API docs
  http://esp-idf.readthedocs.io/en/latest/api-reference/system/deep_sleep.html
  Left the line commented as an example of how to configure peripherals.
  The line below turns off all RTC peripherals in deep sleep.
  */
  //esp_deep_sleep_pd_config(ESP_PD_DOMAIN_RTC_PERIPH, ESP_PD_OPTION_OFF);
  //Serial.println("Configured all RTC Peripherals to be powered down in sleep");

  /*
  Now that we have setup a wake cause and if needed setup the
  peripherals state in deep sleep, we can now start going to
  deep sleep.
  In the case that no wake up sources were provided but deep
  sleep was started, it will sleep forever unless hardware
  reset occurs.
  */
  rtc_gpio_hold_en(EN5V); // enable hold for the RTC GPIO port
  Serial.println("Going to sleep now");
  delay(1000);
  Serial.flush(); 
  //esp_deep_sleep_start();
  #ifdef sigfox
  ESP.deepSleep(SLEEP_TIME * uS_TO_S_FACTOR);
  
  #else
    ESP.deepSleep(postingInterval * 1000);
  #endif
}

void loop() {
  // put your main code here, to run repeatedly:
}
// *****************************************************************
// **   Sigfox Module functions ************************************
// *****************************************************************
// This function wakes up the module before sending any command
void sendWakeUp() {
  Serial.println("Sending WakeUp signal");
  txBuffer[0] = 0x02; // Length
  txBuffer[1] = 0x01; // WakeUp code
  sendDataSPI(2);
  delay(100); // device should move to ready state 100 ms after wakeup
}


// This function puts module to sleep, should be used after each transmission to save battery
void sendToSleep() {
  Serial.println("Sending Sleep signal");
  txBuffer[0] = 0x02; // Length
  txBuffer[1] = 0x03; // Send to Sleep code
  
  sendDataSPI(2);
  delay(50);
  /*recvDataSPI();

  errorCode = rxBuffer[1];
  ol2385State = rxBuffer[2];
  if (errorCode == 0)
    Serial.println("Module put in deep sleep successful!");
  else {
    Serial.print("SFX_ERR: ");
    printByte(errorCode);
    Serial.println();
  }*/
}


// This function retrieves sigfox ID, PAC and lib version
void getDeviceInfo() {
  uint8_t i;
  Serial.println("Retrieving device info");
  
  txBuffer[0] = 0x02; // Length
  txBuffer[1] = 0x08; // DeviceInfo code
  sendDataSPI(2);
  delay(5);
  recvDataSPI();
  
  errorCode = rxBuffer[1];
  ol2385State = rxBuffer[2];
  Serial.print("Device ID: ");
  for (i=4; i>0; i--)
    printByte(rxBuffer[i+2]);
  Serial.println();
  Serial.print("Device PAC: ");
  for (i=8; i>0; i--)
    printByte(rxBuffer[i+6]);
  Serial.println();
  Serial.print("Library version: ");
  for (i=11; i>0; i--)
    printByte(rxBuffer[i+14]);
  Serial.println();
}


// Set Radio Zone 1 (ETSI)
void setRCZ1() {
  //uint8_t i;
  Serial.println("Setting RCZ1");
  
  txBuffer[0] = 0x06; // Length
  txBuffer[1] = 21; // RCZ1 code
  txBuffer[2] = 0;  //This byte contains the amount of 0.25dB steps that needs to be substarcted from TX power (Default value is 0) 
  txBuffer[3] = 1;  //This byte contains the info about which type of XTAL is used on the device(0 for 55.2MHz or 1 for 27.6 MHz TCXO) (default value is 1)
  txBuffer[4] = 3;  //This byte is amount of transmissions that will happen on one time invoking of Send Payload SPI command. By default the value is 3. A value of 0 will lead to 1 TX. All other values will lead to 3 TX.
  txBuffer[5] = 0;  //This byte contains the info about which type of internal PA is used on the device(0-14dBm PA; 1-0dBm PA) (default value is 0)
  sendDataSPI(6);
  delay(5);
  recvDataSPI();
  
  errorCode = rxBuffer[1];
  ol2385State = rxBuffer[2];
}


// Send single bit on sigfox network
void sendBit() {
  Serial.println("Sending single bit");
  txBuffer[0] = 0x02; // Length
  txBuffer[1] = 0x05; // Send bit code
  
  sendDataSPI(2);
  delay(5);
  recvDataSPI();
  
  errorCode = rxBuffer[1];
  ol2385State = rxBuffer[2];
  if (errorCode == 0)
    Serial.println("Bit transmission successful!");
  else {
    Serial.print("SFX_ERR: ");
    printByte(errorCode);
    Serial.println();
  }
}

// This function sends a payload on sigfox network (UL only)
/*void sendPayload(uint8_t * payload, uint8_t payloadLength) {
  sendPayload(payload, payloadLength, 0);
}*/

// This function sends a payload on sigfox network
void sendPayload(uint8_t * payload, uint8_t payloadLength, uint8_t downlink) {
  uint8_t i;
  
  Serial.print("Sending payload (length=");
  Serial.print(payloadLength);
  Serial.print("): ");

  // Building SPI command
  txBuffer[0] = payloadLength + 2; // Length
  txBuffer[1] = 0x04; // sendPayload code
  for (i=0; i<payloadLength; i++) {
    txBuffer[i+2] = payload[i];
    printByte(payload[i]);
  }
  Serial.println();
  
  if (downlink == 1) { // ack requested => command code = 0x07
    Serial.println("Ack requested");
    txBuffer[1] = 0x07;
  }

  // Send and receive data on SPI
  sendDataSPI(payloadLength+2);
  Serial.println("Waiting for response... ");
  recvDataSPI();
  errorCode = rxBuffer[1];
  ol2385State = rxBuffer[2];

  if (errorCode == 0) {
    Serial.println("Transmission successful!");
    Serial.print("Received Payload: ");
    for (i=3; i<rxBuffer[0]; i++)
      printByte(rxBuffer[i]);
    Serial.println();
  }
  else {
    Serial.print("SFX_ERR: ");
    printByte(errorCode);
    Serial.println();
  }

}


/// 
// * Receive SPI data
// * Protocol description: Wait until ACK pin is low and CS high -> Set SPI CS pin low -> Receive first byte with frame length
// * -> Receive rest of frame (length - 1 byte) -> Wait until ACK pin is high -> Set SPI CS pin high 
// 
void recvDataSPI() {
  uint8_t i;

  // Ack pin will go low to indicate rx
  while(digitalRead(ack_Fox) == HIGH) {
    delay(1);
  }
  // set CS pin low
  digitalWrite(cs_Fox, LOW);
  // first byte contains buffer length
  rxBuffer[0] = SPI.transfer(0x00);
  // Start data reception
  for (i=0; i<rxBuffer[0]-1; i++) {
   rxBuffer[i+1] = SPI.transfer(0x00);
   }
  
  while(digitalRead(ack_Fox) == LOW) {
    delay(1);
  }
   delay(1);
 
  // take the chip select high to de-select:
  digitalWrite(cs_Fox, HIGH);
  
}


// Send SPI data
//  * Protocol description: Wait until ACK pin is high and CS high -> Set SPI CS pin low -> Wait until ACK pin is low
//  * -> Send data via SPI -> Wait until ACK pin is high -> Set SPI CS pin high
// /
void sendDataSPI(uint8_t bufLength) {
  uint8_t i;
  
  // Ack pin must be high before starting the transmission protocol
  while(digitalRead(ack_Fox) == LOW) {
    delay(1);
  }
  // set CS pin low
  digitalWrite(cs_Fox, LOW);
  // wait for ack pin low
  while(digitalRead(ack_Fox) == HIGH) {
    delay(1);
  }
    delay(1);

  // Start data transmission
  
  for (i=0; i<bufLength; i++) {
    SPI.transfer(txBuffer[i]);
    
  }
  

  // wait for ack pin high
  while(digitalRead(ack_Fox) == LOW) {
    delay(1);
  }
  delay(1);
  
  // take the chip select high to de-select:
  digitalWrite(cs_Fox, HIGH);

}
// this function prints a byte in hex format as %02x
void printByte(uint8_t b) {
  if (b < 0 || b > 255)
    Serial.print("BYTE_ERR");
  else {
    if (b < 16) {
      Serial.print("0");
      Serial.print(b, HEX);
    }
    else
      Serial.print(b, HEX);
  }
}
// *********  Procedure for connecting to the opensensemap server ***** 
// ********* and transmitting measurement data  *******
void postFloatValue (float measurement, int aftercomma, String sensorId) {
  //Float to String conversion
  char obs[10];
  dtostrf(measurement, 5, aftercomma, obs);
  //create of the json format
  String jsonValue = "{\"value\":";
  jsonValue += obs;
  jsonValue += "}";
  //Connect to OSeM Server and perform POST operation
  
  Serial.println("-------------------------------------");
  Serial.print("Connecting to OSeM Server...");
  if (client.connect(server, 80)) {
    Serial.println("connected!");
    Serial.println("-------------------------------------");
    //build http header
    client.print("POST /boxes/"); client.print(SENSEBOX_ID); client.print("/"); client.print(sensorId); client.println(" HTTP/1.1");
    client.print("Host:");
    client.println(server);
    client.println("Content-Type: application/json");
    client.println("Connection: close");
    client.print("Content-Length: "); client.println(jsonValue.length());
    client.println();
    //Daten senden
    client.println(jsonValue);
  } else {
    Serial.println("failed!");
    Serial.println("-------------------------------------");
  }
  //Display response from server in serial monitor
  waitForServerResponse();
}

void waitForServerResponse (void) {
  //Output incoming bytes
  boolean repeat = true;
  do {
    if (client.available()) {
      char c = client.read();
      Serial.print(c);
    }
    //Terminate connection
    if (!client.connected()) {
      Serial.println();
      Serial.println("--------------");
      Serial.println("Disconnecting.");
      Serial.println("--------------");
      client.stop();
      repeat = false;
    }
  } while (repeat);
}
