#include <Arduino.h>
#include <SigFox.h>          //Include Sigfox-Library
#include <Wire.h>
#include <SPI.h>
#include <Adafruit_BMP3XX.h>
//#include <T9602.h>
#include <Adafruit_AM2315.h>
#include "WindSensor.h"
#include <ArduinoLowPower.h>
#include <TinyGPS++.h>
#include "Adafruit_TSL2591.h"
#include <Adafruit_VEML6070.h>


#define SEALEVELPRESSURE_HPA (1013.25)
#define com3v3  3       //D3 used to connect 3v3
#define Wspeed  0       //Wind Speed Pin D0
#define Wdir    A5      //Analog Wind Direction Pin
#define Ubat    A1      //Analog battery voltage Pin
#define LED 6         // This is the built_in led

#define DEBUG

Adafruit_BMP3XX bmp388; // I2C388;
Adafruit_AM2315 am2315;//T9602 humsens;
WindSensor ws(Wspeed, Wdir);
TinyGPSPlus gps; 
Adafruit_VEML6070 uv = Adafruit_VEML6070();
Adafruit_TSL2591 tsl = Adafruit_TSL2591();
 
double latitude, longitude, alt;
unsigned long nbre_sat;
float temperature, humidity, WinSpeed, alti; 
uint16_t Lux, UV_Ratio;
float sumtemp, sumpres, sumhumi, sumWindSpeed, sumalti, sumUV, sumLux;
int i;
uint32_t unixtime, pascals, Vbatt;
long lastSecond; //The millis counter to see when a second rolls by
volatile long lastWindIRQ = 0;
volatile byte windClicks = 0;
long lastWindCheck = 0;
int windDir = 0;
float windDirAvg = 0;
bool isGPS = false;

typedef struct  { 
  uint8_t Second; 
  uint8_t Minute; 
  uint8_t Hour; 
  uint8_t Wday;   // day of week, sunday is day 1
  uint8_t Day;
  uint8_t Month; 
  uint16_t Year;   // in 4 digits 
} 	timedate;

void wspeedIRQ(void);
float get_wind_speed(void);
unsigned long unixTimestamp(int year, int month, int day,
              int hour, int min, int sec);
void displayInfo(void);
uint8_t sigfoxdata[14] = {0x48, 0x65, 0x6c, 0x6c, 0x6f, 0x20, 0x53, 0x69, 0x67, 0x66, 0x6f, 0x78}; // "Hello sigfox"
timedate TD;

void setup() {
 pinMode(com3v3, OUTPUT);
 pinMode(LED,OUTPUT);
 digitalWrite(LED,LOW);
 analogReadResolution(10);
 analogReference(AR_INTERNAL1V65);
#ifdef DEBUG
 Serial.begin(9600);
#endif
 //while (!Serial);
 Serial1.begin(9600);  //
 //while (!Serial1);
// ********************************************************** //
// * For the Sigfox-Modem *********************************** //
// ********************************************************** //

  // Check if Sigfox modem is present and can be initialized.
  // Output error if modem does not exist or not ok
  if (!SigFox.begin()) //Initialize Sigfox modem
  {
    #ifdef DEBUG
    Serial.println("Sigfox modem not found! - rebooting!");
    #endif
    NVIC_SystemReset();
    while (1);  
  }
  else
  {
    #ifdef DEBUG
    Serial.println("Sigfox-Modem Init OK !\n");
    #endif
  }
  SigFox.reset();
  delay(100);
  SigFox.debug();
  SigFox.end();
  #ifdef DEBUG
  Serial.println("Sigfox OK !\n");
  #endif
  // ************************************************************ //
  // * For the sensors ****************************************** //
  // ************************************************************ 0//
 // input from wind meters windspeed sensor
  ws.initWindSensor();
  // attach external interrupt pins to IRQ functions
	attachInterrupt(Wspeed, wspeedIRQ, FALLING);
  digitalWrite(com3v3, LOW);
  delay(500);
   
  do{
    if (Serial1.available() > 0)
    if (gps.encode(Serial1.read()))
    #ifdef DEBUG
      displayInfo();
    #endif  
      delay(1000);
  }while(!gps.location.isValid()&&millis() <= 5000);
   

  // If 5000 milliseconds pass and there are no characters coming in
  // over the software serial port, show a "No GPS detected" error
  if (millis() > 5000 && gps.charsProcessed() < 10)
  {
    #ifdef DEBUG
    Serial.println("No GPS detected");
    #endif
    isGPS = false;
  }
  else isGPS = true;
  if (!bmp388.begin_I2C()) {
    #ifdef DEBUG
    Serial.println("Could not find a valid BMP3 sensor, check wiring!");
    #endif
    while (1);
  }
  
  //humsens.begin();
  if (! am2315.begin()) {
    #ifdef DEBUG
     Serial.println("Sensor not found, check wiring & pullups!");
    #endif
     while (1);
  }
  // Set up oversampling and filter initialization
  bmp388.setTemperatureOversampling(BMP3_OVERSAMPLING_8X);
  bmp388.setPressureOversampling(BMP3_OVERSAMPLING_4X);
  bmp388.setIIRFilterCoeff(BMP3_IIR_FILTER_COEFF_3);
  //bmp.setOutputDataRate(BMP3_ODR_50_HZ);
  if (tsl.begin()) 
  {
    Serial.println(F("Found a TSL2591 sensor"));
  } 
  else 
  {
    Serial.println(F("No sensor found ... check your wiring?"));
    while (1);
  }
  tsl.setGain(TSL2591_GAIN_LOW);    // 1x gain (bright light)
  tsl.setTiming(TSL2591_INTEGRATIONTIME_200MS);
  uv.begin(VEML6070_1_T);  // pass in the integration time constant
  #ifdef DEBUG
  Serial.println("Sensors OK !\n");
  #endif
  // turn on interrupts
	interrupts();
  // We need to have to time to program the Arduino after a reset
  // Otherwise it does not respond when in low-power mode (USB disconnected...)
  #ifdef DEBUG
   Serial.println("Booting...");
  #endif
   digitalWrite(LED,HIGH);
   delay(5000);
   digitalWrite(LED,LOW);
}

void loop() {
  
  digitalWrite(com3v3, LOW);
  delay(500);
  
  sumpres = 0.0; sumtemp = 0.0 ; sumhumi = 0.0; sumWindSpeed = 0.0; sumalti = 0.0; sumLux = 0.0; sumUV = 0.0;
  // *** Vbatt Measurment Vbatt(mV) = 1000*Ncan*(680+330)/(1024*330)
  Vbatt = ((analogRead(ADC_BATTERY))*(330+680)*165)/(1024*33);
  //Serial.print("Ubatt="); Serial.println(Vbatt);
   if (! bmp388.performReading()) {
    Serial.println("Failed to perform reading :(");
    return;
    }
  delay(1000);
  pascals = (uint32_t)bmp388.pressure; // The first read is not correct
  for(i = 0; i<10; i++){
    
    if (! bmp388.performReading()) {
    //Serial.println("Failed to perform reading :(");
    return;
    }
    // Temperature measurement
    //temperature = humsens.GetTemperature();
    // Humidity measurement
    //humidity = humsens.GetHumidity();
    /*if (! am2315.readTemperatureAndHumidity(&temperature, &humidity)) {
    Serial.println("Failed to read data from AM2315");
    //return;
  }*/
  am2315.readTemperatureAndHumidity(&temperature, &humidity);
   //pressure measurement
    pascals = (uint32_t)bmp388.pressure; 
  // Altitude measurement
    alti = bmp388.readAltitude(SEALEVELPRESSURE_HPA);
  // UV Ratio measurement
    UV_Ratio = uv.readUV();
  //Lux measurement
    Lux = tsl.getLuminosity(TSL2591_VISIBLE);
  //Wind Speed
    float currentSpeed = get_wind_speed();
    ws.determineWindDir();
    windDir = ws.getWindDir();
    //average of the 10 measurements 
    sumtemp = sumtemp + temperature ;
    sumpres = sumpres + pascals; 
    sumalti = sumalti + alti;
    sumhumi = sumhumi + humidity;
    sumLux = sumLux + (float)Lux;
    sumUV = sumUV + (float)UV_Ratio;
    sumWindSpeed = sumWindSpeed + currentSpeed;   
    delay(1000);
    digitalWrite(LED,HIGH);
  }
  temperature = sumtemp/10;
  pascals = sumpres/10;
  alti = sumalti/10;
  humidity = sumhumi/10;
  WinSpeed = sumWindSpeed/10;
  Lux = sumLux/10;
  UV_Ratio = sumUV/10;
  windDirAvg = ws.getWindDirAvg();
  // *************  GPS Data  **********************************
    if (isGPS){
      do{
          if (Serial1.available()) 
          gps.encode(Serial1.read());
    }while (!gps.location.isUpdated());
    
      latitude = gps.location.lat();
      longitude = gps.location.lng();
      alt = gps.altitude.meters();
      nbre_sat = gps.satellites.value();
      TD.Day = gps.date.day();
      TD.Month = gps.date.month();
      TD.Year = gps.date.year();
      TD.Hour = gps.time.hour();
      TD.Minute = gps.time.minute();
      TD.Second = gps.time.second();
      unixtime = unixTimestamp(TD.Year, TD.Month, TD.Day, TD.Hour, TD.Minute, TD.Second);
    }
   
 
     /* Serial.println("-------- FIX GPS ------------");
      Serial.print("LATITUDE="); Serial.println(latitude);
      Serial.print("LONGITUDE="); Serial.println(longitude);
      Serial.print("ALTITUDE (m) ="); Serial.println(alt);
      
      Serial.print("NOMBRE SATTELLITES="); Serial.println(nbre_sat);
    */
  
  // ******************************************************** //
  
  digitalWrite(com3v3, HIGH);
  
  #ifdef DEBUG
  Serial.print("Temp : ");
  Serial.print(temperature);
  Serial.println(" ºC");
  Serial.print("Pres : ");
  Serial.print(pascals); Serial.println(" Pa");
  Serial.print("Hum : ");
  Serial.print(humidity); Serial.println(" %");
  Serial.print("Approx. Altitude = ");
  Serial.print(alti); Serial.println(" m");
  Serial.print("Wind Speed : ");
  Serial.print(WinSpeed);  Serial.println(" km/h");
  Serial.println();
  Serial.print("Wind Direction : ");
  Serial.print((uint16_t)windDirAvg);
  Serial.println(" Degres");
  Serial.println();
  Serial.print("Luminosity : ");
  Serial.print((uint16_t)Lux);
  Serial.println(" LUX");
  Serial.println();
  Serial.print("UV Ratio : ");
  Serial.print((uint16_t)UV_Ratio);
  Serial.println();
  #endif
  sigfoxdata[0] = (uint8_t)((uint16_t)(temperature*10)>>8);
	sigfoxdata[1] = (uint8_t)(temperature*10);
	sigfoxdata[2] = (uint8_t)((uint32_t)pascals>>16);
	sigfoxdata[3] = (uint8_t)((uint32_t)pascals>>8);
	sigfoxdata[4] = (uint8_t)pascals;
	sigfoxdata[5] = (uint8_t)humidity;
  sigfoxdata[6] = (uint8_t)WinSpeed;
  //sigfoxdata[7] = (uint8_t)((uint16_t)windDirAvg>>8);
	//sigfoxdata[8] = (uint8_t)windDirAvg;
  //sigfoxdata[9] = (uint8_t)((uint16_t)alti>>8);
	//sigfoxdata[10] = (uint8_t)alti;
  sigfoxdata[7] = (uint8_t)((uint16_t)Lux>>8);
	sigfoxdata[8] = (uint8_t)Lux;
  sigfoxdata[9] = (uint8_t)((uint16_t)UV_Ratio>>8);
	sigfoxdata[10] = (uint8_t)UV_Ratio;
	sigfoxdata[11] = (uint8_t)(Vbatt/100);


  SigFox.begin();// Wait at least 30ms after first configuration (100ms before)
  delay(100);
  SigFox.status();
  SigFox.beginPacket();
  SigFox.write(sigfoxdata,12);
  SigFox.endPacket(false);
  SigFox.end();

   // Wait for 12 minutes.
   // Low Power version - be carefull of bug
   LowPower.deepSleep(12*60*1000);
   
   // Normal version*/
   //delay(1*60*1000);
   
}
//************************** 
// interrupt function for the anamometer
// Activated by the magnet in the anemometer (2 ticks per rotation), attached to input Wspeed  IO32
//**************************
void wspeedIRQ(void)

{
	if (millis() - lastWindIRQ > 10) // Ignore switch-bounce glitches less than 10ms (142MPH max reading) after the reed switch closes
	{
		lastWindIRQ = millis(); //Grab the current time
		windClicks++; //There is 1.492MPH for each click per second.
	}
}
//***************
//Returns the instataneous wind speed
//***************
float get_wind_speed(void)
{
	float deltaTime = millis() - lastWindCheck; 

	deltaTime /= 1000.0; //Convert to seconds

	float windSpeed = (float)windClicks / deltaTime; 

	windClicks = 0; //Reset and start watching for new wind
	lastWindCheck = millis();

	windSpeed = 2.4 * windSpeed; // 1 pulse per seconds --> 2,4 km/h

	return(windSpeed);
}
// *********************************************************************
// *** Convertion date/time --> Unix Timestamp *************************
// *********************************************************************
unsigned long unixTimestamp(int year, int month, int day,
              int hour, int min, int sec)
{
  const short days_since_beginning_of_year[12] = {0,31,59,90,120,151,181,212,243,273,304,334};

  int leap_years = ((year-1)-1968)/4
                  - ((year-1)-1900)/100
                  + ((year-1)-1600)/400;

  long days_since_1970 = (year-1970)*365 + leap_years
                      + days_since_beginning_of_year[month-1] + day-1;

  if ( (month>2) && (year%4==0 && (year%100!=0 || year%400==0)) )
    days_since_1970 += 1; /* +leap day, if year is a leap year */

  return sec + 60 * ( min + 60 * (hour + 24*days_since_1970) );
}
// ********************************************************************
void displayInfo(void)
{
  if (gps.location.isValid())
  {
    Serial.print("Latitude: ");
    Serial.println(gps.location.lat(), 6);
    Serial.print("Longitude: ");
    Serial.println(gps.location.lng(), 6);
    Serial.print("Altitude: ");
    Serial.println(gps.altitude.meters());
  }
  else
  {
    Serial.println("Location: Not Available");
  }
  
  Serial.print("Date: ");
  if (gps.date.isValid())
  {
    Serial.print(gps.date.month());
    Serial.print("/");
    Serial.print(gps.date.day());
    Serial.print("/");
    Serial.println(gps.date.year());
  }
  else
  {
    Serial.println("Not Available");
  }

  Serial.print("Time: ");
  if (gps.time.isValid())
  {
    if (gps.time.hour() < 10) Serial.print(F("0"));
    Serial.print(gps.time.hour());
    Serial.print(":");
    if (gps.time.minute() < 10) Serial.print(F("0"));
    Serial.print(gps.time.minute());
    Serial.print(":");
    if (gps.time.second() < 10) Serial.print(F("0"));
    Serial.print(gps.time.second());
    Serial.print(".");
    if (gps.time.centisecond() < 10) Serial.print(F("0"));
    Serial.println(gps.time.centisecond());
  }
  else
  {
    Serial.println("Not Available");
  }

  Serial.println();
  Serial.println();
  
}