/* USER CODE BEGIN Header */
/**
  ******************************************************************************
  * @file           : main.c
  * @brief          : Main program body
  ******************************************************************************
  * @attention
  *
  * <h2><center>&copy; Copyright (c) 2020 STMicroelectronics.
  * All rights reserved.</center></h2>
  *
  * This software component is licensed by ST under BSD 3-Clause license,
  * the "License"; You may not use this file except in compliance with the
  * License. You may obtain a copy of the License at:
  *                        opensource.org/licenses/BSD-3-Clause
  *
  ******************************************************************************
  */
/* USER CODE END Header */

/* Includes ------------------------------------------------------------------*/
#include "main.h"

/* Private includes ----------------------------------------------------------*/
/* USER CODE BEGIN Includes */
#include "MPL3115A2.h"
#include "GPS.h"
#include <stdio.h>
#include <string.h>
/* USER CODE END Includes */

/* Private typedef -----------------------------------------------------------*/
/* USER CODE BEGIN PTD */
typedef struct{

	float temperature1;
	float humidity;
	float pressure;
	uint8_t wind_dir;
	float wind_speed;
	uint16_t part10;
	uint16_t part25;
	uint16_t part10env;
	uint16_t part25env;
}mesurescapteurs;

typedef struct {
    uint16_t framelen;
    uint16_t pm10_standard, pm25_standard, pm100_standard;
    uint16_t pm10_env, pm25_env, pm100_env;
    uint16_t particles_03um, particles_05um, particles_10um, particles_25um, particles_50um, particles_100um;
    uint16_t unused;
    uint16_t checksum;
} PM25data;


/* USER CODE END PTD */

/* Private define ------------------------------------------------------------*/
/* USER CODE BEGIN PD */
#define adr_T9602 0x50  //0x50 in 8bits mode or 0x28 in 7bits mode
#define MPLAddressI2C 0xC0;  //This is the slave address of the device
#define adr_bat 0x64  //0x64 in 8bits mode or 0x32 in 7bits mode
#define false 0
#define true 1
//#define xbee
#define sigfox

/* USER CODE END PD */

/* Private macro -------------------------------------------------------------*/
/* USER CODE BEGIN PM */

/* USER CODE END PM */

/* Private variables ---------------------------------------------------------*/
ADC_HandleTypeDef hadc;

I2C_HandleTypeDef hi2c1;

RTC_HandleTypeDef hrtc;

TIM_HandleTypeDef htim21;

UART_HandleTypeDef huart1;
UART_HandleTypeDef huart2;
UART_HandleTypeDef huart5;

/* USER CODE BEGIN PV */
char msg[150], askmes=0, validSend;
char datain[5];
uint32_t TimeStamp, lastWindCheck; //Mes_delay, send_delay
uint16_t seconds = 30, nmes, nextsec = 32;
mesurescapteurs captmes;
mesurescapteurs meanmes;
/*const float tab_dirvent[16][2] = {
    {0.0, 3057}, {22.5, 2355}, {45.0, 2484}, {67.5, 1264},
    {90.0, 1307}, {112.5, 1178}, {135.0, 1684}, {157.5, 1452},
    {180.0, 2029}, {202.5, 1892}, {225.0, 2818}, {247.5, 2763},
    {270.0, 3195}, {292.5, 3118}, {315.0, 3264}, {337.5, 2937}
};*/
const float tab_dirvent[16][2] = {
    {0, 3057}, {1, 2355}, {2, 2484}, {3, 1264},
    {4, 1307}, {5, 1178}, {6, 1684}, {7, 1452},
    {8, 2029}, {9, 1892}, {10, 2818}, {11, 2763},
    {12, 3195}, {13, 3118}, {14, 3264}, {15, 2937}
};
uint16_t valCAN, compt_anemo;
RTC_TimeTypeDef time;
RTC_DateTypeDef date;
float windspeedsum, tempersum, pressuresum, humitysum;
uint32_t pm10sum, pm25sum, pm10envsum, pm25envsum;
uint8_t sigfoxdata[12];
uint8_t  Nbr, data = '\0', pmbuffer[32] = {0};
volatile uint8_t go = false, pret = false ;

/* USER CODE END PV */

/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config(void);
static void MX_GPIO_Init(void);
static void MX_ADC_Init(void);
static void MX_I2C1_Init(void);
static void MX_RTC_Init(void);
static void MX_USART1_UART_Init(void);
static void MX_USART2_UART_Init(void);
static void MX_USART5_UART_Init(void);
static void MX_TIM21_Init(void);
/* USER CODE BEGIN PFP */
void delai_ms(uint16_t duree);
void mesures(mesurescapteurs *mescapt);
uint8_t dir_vent(float Vin);
float calcVin (void);
unsigned long unixTimestamp(int year, int month, int day,
              int hour, int min, int sec);

/* USER CODE END PFP */

/* Private user code ---------------------------------------------------------*/
/* USER CODE BEGIN 0 */
void delai_ms(uint16_t duree){
	uint8_t for1ms;
	for(;duree!=0;duree--){
		for(for1ms=0; for1ms<128; for1ms++);
	}
}
unsigned long unixTimestamp(int year, int month, int day,
              int hour, int min, int sec)
{
  const short days_since_beginning_of_year[12] = {0,31,59,90,120,151,181,212,243,273,304,334};

  int leap_years = ((year-1)-1968)/4
                  - ((year-1)-1900)/100
                  + ((year-1)-1600)/400;

  long days_since_1970 = (year-1970)*365 + leap_years
                      + days_since_beginning_of_year[month-1] + day-1;

  if ( (month>2) && (year%4==0 && (year%100!=0 || year%400==0)) )
    days_since_1970 += 1; /* +leap day, if year is a leap year */

  return sec + 60 * ( min + 60 * (hour + 24*days_since_1970) );
}
//******************************************
//Particles sensor callback
//******************************************
void pms5003Read(void)
{
    if (go) {
        pmbuffer[Nbr++] = data;
        if(Nbr>=32) {
            pret = true;
            Nbr=0;
            go = false;
        }
    } else {
        if( data == 0x42 && !pret) {
            go = true;
            pmbuffer[Nbr++] = data;
        }
    }
    HAL_UART_Receive_IT(&huart1,&data,1);
}
//******************************************
// Measures proceed
//******************************************
void mesures(mesurescapteurs *mescapt)
{
    uint8_t wake_cmd[1];
    uint8_t data[4];
    MPL_Read_MPL_Data();
    HAL_Delay(100);
//****  Digital Humidity Sensor T9602 *****
    data[0] = 0x00;
    data[1] = 0x00;
    data[2] = 0x00;
    data[3] = 0x00;
    wake_cmd[0] = 0x00;
    HAL_I2C_Master_Transmit(&hi2c1, adr_T9602, wake_cmd, 1, 1000);//i2c.write(adr_T9602, wake_cmd, 1);
    HAL_Delay(50);
    HAL_I2C_Master_Receive(&hi2c1, adr_T9602, data, 4, 1000);//i2c.read(adr_T9602, data, 4);

    mescapt->humidity = ((((uint16_t)data[0] & 63) << 8) + (uint16_t)data[1]) / 163.84;
    mescapt->temperature1 = (((data[2] << 6) + (data[3] / 4)) / 99.29) - 40;
    mescapt->pressure = MPL_Pressure();
    mescapt->wind_dir = dir_vent(calcVin());


}
//****************************************************
//   UART Interruptions
//***************************************************
void HAL_UART_RxCpltCallback(UART_HandleTypeDef *huart){

	if (huart->Instance == USART5)  //UART for serial XBee
	    {

			if (strcmp(datain, "dem1")==0){
				//sprintf(msg,"{%.2f,%.2f,%.2f,%.2f,%.2f}",captmes.temperature1, captmes.temperature2, captmes.humidity, captmes.pressure, captmes.wind_dir);//sprintf(msg,"{%.2f,%.2f,%.2f,%.2f,%.2f}",captmes.temperature1,captmes.temperature2, captmes.humidity, captmes.pressure, captmes.wind_dir );
			 	//HAL_UART_Transmit_IT(&huart5,(uint8_t*)msg, strlen(msg) );
				askmes = 1;
			 	datain[0] = 0;
			 	HAL_UART_Receive_IT(&huart5,(uint8_t*)datain, 4 );
			  }
			else {
				askmes = 0;
				datain[0] = 0;
				HAL_UART_Receive_IT(&huart5,(uint8_t*)datain, 4 );
			}
	    }
	if (huart->Instance == USART2)  //UART for GPS
		{
			GPS_CallBack();
		}
	if (huart->Instance == USART1)  //UART for Particles sensor
		{
			pms5003Read();
		}

}
//****************************************************
//   Wind_Speed Interruption
//***************************************************
void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin){
	if (GPIO_Pin == GPIO_PIN_5)
	    {
			compt_anemo++;
	    }
}
//****************************************************
//		Wind direction calculation
//****************************************************
uint8_t dir_vent(float Vin)
{
    char i;
    float wd;
    for (i = 0; i < 16; i ++) {
        if (Vin > tab_dirvent[i][1] * 0.98 && Vin < tab_dirvent[i][1] * 1.02) {
            wd = tab_dirvent[i][0];
            return((uint8_t)wd);
        }
    }
    return -1;
}
float calcVin (void)
{
    float VIN;
    if(HAL_ADC_PollForConversion(&hadc, 100000) == HAL_OK){
    		  VIN = (float)HAL_ADC_GetValue(&hadc);
    		  VIN = VIN*3300/4096;
    		  //rint = rint*3300/(3300-rint);
    }
    return(VIN);
}
//***************************************
//Returns the instataneous wind speed
//**************************************
float get_wind_speed()
{
  float deltaTime = HAL_GetTick() - lastWindCheck;

  deltaTime /= 1000.0; //Convert to seconds

  float windSpeed = (float)compt_anemo / deltaTime; //3 / 0.750s = 4

  compt_anemo = 0; //Reset and start watching for new wind
  lastWindCheck = HAL_GetTick();

  windSpeed = 2.4 * windSpeed; // 1 pulse per seconds --> 2,4 km/h

  return(windSpeed);
}

/* USER CODE END 0 */

/**
  * @brief  The application entry point.
  * @retval int
  */
int main(void)
{
  /* USER CODE BEGIN 1 */
	char ind;
	uint16_t buffer16bits[15];
	PM25data pm25data;
  /* USER CODE END 1 */

  /* MCU Configuration--------------------------------------------------------*/

  /* Reset of all peripherals, Initializes the Flash interface and the Systick. */
  HAL_Init();

  /* USER CODE BEGIN Init */

  /* USER CODE END Init */

  /* Configure the system clock */
  SystemClock_Config();

  /* USER CODE BEGIN SysInit */

  /* USER CODE END SysInit */

  /* Initialize all configured peripherals */
  MX_GPIO_Init();
  MX_ADC_Init();
  MX_I2C1_Init();
  MX_RTC_Init();
  MX_USART1_UART_Init();
  MX_USART2_UART_Init();
  MX_USART5_UART_Init();
  MX_TIM21_Init();
  /* USER CODE BEGIN 2 */
  MPL_Init();

  HAL_GPIO_WritePin(EN_GPS_GPIO_Port, EN_GPS_Pin, GPIO_PIN_SET);
  //HAL_GPIO_WritePin(EN_Part_GPIO_Port, EN_Part_Pin, GPIO_PIN_SET);
  //HAL_GPIO_WritePin(Sleep_Fox_GPIO_Port, Sleep_Fox_Pin, GPIO_PIN_RESET);

  GPS_Init();

  do{
  	  GPS_Process();
  	  HAL_Delay(100);
    }while(GPS.GPGGA.PositionFixIndicator !=1);//while(GPS.GPGGA.status !='A');
  time.Hours = GPS.GPGGA.UTC_Hour ;
  time.Minutes = GPS.GPGGA.UTC_Min;
  time.Seconds = GPS.GPGGA.UTC_Sec;
  date.Year = GPS.GPGGA.gps_year; 	//date.Year = 20;
  date.Month = GPS.GPGGA.gps_month;	//date.Month = 2;
  date.Date = GPS.GPGGA.gps_day;	// date.Date = 19;
  HAL_RTC_SetTime(&hrtc, &time, RTC_FORMAT_BIN); //mettre la RTC à l’heure "time"
  HAL_RTC_SetDate(&hrtc, &date, RTC_FORMAT_BIN);
  HAL_ADC_Start(&hadc);
  //HAL_TIM_Base_Start_IT(&htim21);
  compt_anemo = 0;
  windspeedsum = 0; tempersum = 0;  pressuresum = 0; humitysum = 0;
  nmes = 0; validSend = 0;
#ifdef xbee
  HAL_UART_Receive_IT(&huart5,(uint8_t*)datain, 4 );
#endif
  HAL_GPIO_WritePin(EN_GPS_GPIO_Port, EN_GPS_Pin, GPIO_PIN_RESET); // GPS Halted
  /* USER CODE END 2 */

  /* Infinite loop */
  /* USER CODE BEGIN WHILE */
  while (1)
  {
	  if(askmes ==1 ||validSend==1){
		  HAL_TIM_Base_Start_IT(&htim21);
	  HAL_GPIO_WritePin(EN_Part_GPIO_Port, EN_Part_Pin, GPIO_PIN_SET);
	  //HAL_GPIO_WritePin(Sleep_Fox_GPIO_Port, Sleep_Fox_Pin, GPIO_PIN_SET);
	  HAL_UART_Receive_IT(&huart1,&data,1);
	  if(seconds == nextsec){
	  	  		windspeedsum = windspeedsum + get_wind_speed();
	  	  		mesures(&captmes);
	  	  		tempersum = tempersum + captmes.temperature1;
	  	  		pressuresum = pressuresum + captmes.pressure;
	  	  		humitysum = humitysum + captmes.humidity;
	  	  		if(pret) {
	  	  		    for(ind = 0; ind <15; ind++) {
	  	  		    	buffer16bits[ind] = (uint16_t)pmbuffer[2 + 2*ind + 1];
	  	  		    	buffer16bits[ind] = buffer16bits[ind] + ((uint16_t)pmbuffer[2 + 2*ind] << 8);
	  	  		    	}
	  	  		    memcpy((void*)&pm25data, buffer16bits, 30);
	  	  		    pret = false;
	  	  		    }
	  	  		  pm10sum = pm10sum + (uint32_t)pm25data.pm10_standard;
	  	  		  pm25sum = pm25sum + (uint32_t)pm25data.pm25_standard;
	  	  		  pm10envsum = pm10envsum + (uint32_t)pm25data.pm10_env;
	  	  		  pm25envsum = pm25envsum + (uint32_t)pm25data.pm25_env;
	  	  		  nmes++;
	  	  		  nextsec = seconds + 2;
	  	  	  }
	  if(nmes==10){

	  	  		  meanmes.wind_speed = windspeedsum/nmes;
	  	  		  meanmes.humidity = humitysum/nmes;
	  	  		  meanmes.pressure = pressuresum/nmes;
	  	  		  meanmes.temperature1 = tempersum/nmes;
	  	  		  meanmes.part10 = pm10sum/nmes;
	  	  		  meanmes.part25 = pm25sum/nmes;
	  	  		  meanmes.part10env = pm10envsum/nmes;
	  	  		  meanmes.part25env = pm25envsum/nmes;
	  	  		  captmes.wind_dir = dir_vent(calcVin());
	  	  		  HAL_RTC_GetTime(&hrtc, &time, RTC_FORMAT_BIN); //lire heure dans RTC
	  	  		  HAL_RTC_GetDate(&hrtc, &date, RTC_FORMAT_BIN); //lire date dans RTC (non utilisée ici)
#ifdef xbee
	  	  		  sprintf(msg,"T=%.2f,H=%.2f,P=%.2f,D=%.2f,V=%.2f %2d/%2d/%2d %2d:%2d:%2d\n Part10=%d env=%d Part25=%d env=%d\n",meanmes.temperature1, meanmes.humidity, meanmes.pressure, captmes.wind_dir*22.5, meanmes.wind_speed, date.Date, date.Month, date.Year, time.Hours, time.Minutes, time.Seconds, meanmes.part10, meanmes.part10env, meanmes.part25, meanmes.part25env);
	  	  		  HAL_UART_Transmit_IT(&huart5,(uint8_t*)msg, strlen(msg) );
#endif
#ifdef sigfox

	  	  		  TimeStamp = unixTimestamp(2000+date.Year, date.Month, date.Date, time.Hours, time.Minutes, time.Seconds);
	  	  		  //sigfoxdata[0] = (uint8_t)(TimeStamp>>24);
	  	  		  //sigfoxdata[1] = (uint8_t)(TimeStamp>>16);
	  	  		  //sigfoxdata[2] = (uint8_t)(TimeStamp>>8);
	  	  		  //sigfoxdata[3] = (uint8_t)TimeStamp;
	  	  		  sigfoxdata[0] = (uint8_t)((uint16_t)(meanmes.temperature1*10)>>8);
	  	  		  sigfoxdata[1] = (uint8_t)(meanmes.temperature1*10);
	  	  		  sigfoxdata[2] = (uint8_t)((uint32_t)meanmes.pressure>>16);
	  	  		  sigfoxdata[3] = (uint8_t)((uint32_t)meanmes.pressure>>8);
	  	  		  sigfoxdata[4] = (uint8_t)meanmes.pressure;
	  	  		  sigfoxdata[5] = (uint8_t)meanmes.humidity;
	  	  		  sigfoxdata[6] = (uint8_t)((uint16_t)(meanmes.part10env)>>8);
	  	  		  sigfoxdata[7] = (uint8_t)(meanmes.part10env);
	  	  		  sigfoxdata[8] = (uint8_t)((uint16_t)(meanmes.part25env)>>8);
	  	  		  sigfoxdata[9] = (uint8_t)(meanmes.part25env);
	  	  		  sigfoxdata[10] = (uint8_t)meanmes.wind_speed;
	  	  		  sigfoxdata[11] = (uint8_t)captmes.wind_dir;
		  //sprintf(msg,"{%.2f,%.2f,%.2f,%.2f,%.2f time: %2d:%2d:%2d}",captmes.temperature1, captmes.humidity, captmes.pressure, captmes.wind_dir, captmes.wind_speed, time.Hours, time.Minutes, time.Seconds);
	  	  		  HAL_UART_Transmit_IT(&huart5,(uint8_t*)sigfoxdata, 12 );

#endif
		  	  	  HAL_GPIO_WritePin(EN_Part_GPIO_Port, EN_Part_Pin, GPIO_PIN_RESET);
		  	  	  //HAL_GPIO_WritePin(Sleep_Fox_GPIO_Port, Sleep_Fox_Pin, GPIO_PIN_RESET);
		  	  	  HAL_TIM_Base_Stop_IT(&htim21);
	  	  		  seconds = 30;
	  	  		  nextsec = 32;
	  	  		  nmes = 0;
	  	  		  windspeedsum = 0; tempersum = 0;  pressuresum = 0; humitysum = 0;
	  	  		  pm10sum = 0; pm25sum = 0; pm10envsum = 0; pm25envsum = 0;
	  	  		  askmes = 0;
	  	  		  validSend = 0;

	  	  	  }

	  }
    /* USER CODE END WHILE */

    /* USER CODE BEGIN 3 */
  }
  /* USER CODE END 3 */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void SystemClock_Config(void)
{
  RCC_OscInitTypeDef RCC_OscInitStruct = {0};
  RCC_ClkInitTypeDef RCC_ClkInitStruct = {0};
  RCC_PeriphCLKInitTypeDef PeriphClkInit = {0};

  /** Configure the main internal regulator output voltage 
  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE3);
  /** Configure LSE Drive Capability 
  */
  HAL_PWR_EnableBkUpAccess();
  __HAL_RCC_LSEDRIVE_CONFIG(RCC_LSEDRIVE_LOW);
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_LSE|RCC_OSCILLATORTYPE_MSI;
  RCC_OscInitStruct.LSEState = RCC_LSE_ON;
  RCC_OscInitStruct.MSIState = RCC_MSI_ON;
  RCC_OscInitStruct.MSICalibrationValue = 0;
  RCC_OscInitStruct.MSIClockRange = RCC_MSIRANGE_5;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }
  /** Initializes the CPU, AHB and APB busses clocks 
  */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_MSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV1;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }
  PeriphClkInit.PeriphClockSelection = RCC_PERIPHCLK_USART1|RCC_PERIPHCLK_USART2
                              |RCC_PERIPHCLK_I2C1|RCC_PERIPHCLK_RTC;
  PeriphClkInit.Usart1ClockSelection = RCC_USART1CLKSOURCE_PCLK2;
  PeriphClkInit.Usart2ClockSelection = RCC_USART2CLKSOURCE_PCLK1;
  PeriphClkInit.I2c1ClockSelection = RCC_I2C1CLKSOURCE_PCLK1;
  PeriphClkInit.RTCClockSelection = RCC_RTCCLKSOURCE_LSE;
  if (HAL_RCCEx_PeriphCLKConfig(&PeriphClkInit) != HAL_OK)
  {
    Error_Handler();
  }
}

/**
  * @brief ADC Initialization Function
  * @param None
  * @retval None
  */
static void MX_ADC_Init(void)
{

  /* USER CODE BEGIN ADC_Init 0 */

  /* USER CODE END ADC_Init 0 */

  ADC_ChannelConfTypeDef sConfig = {0};

  /* USER CODE BEGIN ADC_Init 1 */

  /* USER CODE END ADC_Init 1 */
  /** Configure the global features of the ADC (Clock, Resolution, Data Alignment and number of conversion) 
  */
  hadc.Instance = ADC1;
  hadc.Init.OversamplingMode = DISABLE;
  hadc.Init.ClockPrescaler = ADC_CLOCK_SYNC_PCLK_DIV1;
  hadc.Init.Resolution = ADC_RESOLUTION_12B;
  hadc.Init.SamplingTime = ADC_SAMPLETIME_12CYCLES_5;
  hadc.Init.ScanConvMode = ADC_SCAN_DIRECTION_FORWARD;
  hadc.Init.DataAlign = ADC_DATAALIGN_RIGHT;
  hadc.Init.ContinuousConvMode = ENABLE;
  hadc.Init.DiscontinuousConvMode = DISABLE;
  hadc.Init.ExternalTrigConvEdge = ADC_EXTERNALTRIGCONVEDGE_NONE;
  hadc.Init.ExternalTrigConv = ADC_SOFTWARE_START;
  hadc.Init.DMAContinuousRequests = DISABLE;
  hadc.Init.EOCSelection = ADC_EOC_SINGLE_CONV;
  hadc.Init.Overrun = ADC_OVR_DATA_PRESERVED;
  hadc.Init.LowPowerAutoWait = DISABLE;
  hadc.Init.LowPowerFrequencyMode = DISABLE;
  hadc.Init.LowPowerAutoPowerOff = DISABLE;
  if (HAL_ADC_Init(&hadc) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure for the selected ADC regular channel to be converted. 
  */
  sConfig.Channel = ADC_CHANNEL_4;
  sConfig.Rank = ADC_RANK_CHANNEL_NUMBER;
  if (HAL_ADC_ConfigChannel(&hadc, &sConfig) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN ADC_Init 2 */

  /* USER CODE END ADC_Init 2 */

}

/**
  * @brief I2C1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_I2C1_Init(void)
{

  /* USER CODE BEGIN I2C1_Init 0 */

  /* USER CODE END I2C1_Init 0 */

  /* USER CODE BEGIN I2C1_Init 1 */

  /* USER CODE END I2C1_Init 1 */
  hi2c1.Instance = I2C1;
  hi2c1.Init.Timing = 0x00000708;
  hi2c1.Init.OwnAddress1 = 0;
  hi2c1.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  hi2c1.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  hi2c1.Init.OwnAddress2 = 0;
  hi2c1.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
  hi2c1.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  hi2c1.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&hi2c1) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Analogue filter 
  */
  if (HAL_I2CEx_ConfigAnalogFilter(&hi2c1, I2C_ANALOGFILTER_ENABLE) != HAL_OK)
  {
    Error_Handler();
  }
  /** Configure Digital filter 
  */
  if (HAL_I2CEx_ConfigDigitalFilter(&hi2c1, 0) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN I2C1_Init 2 */

  /* USER CODE END I2C1_Init 2 */

}

/**
  * @brief RTC Initialization Function
  * @param None
  * @retval None
  */
static void MX_RTC_Init(void)
{

  /* USER CODE BEGIN RTC_Init 0 */

  /* USER CODE END RTC_Init 0 */

  RTC_TimeTypeDef sTime = {0};
  RTC_DateTypeDef sDate = {0};

  /* USER CODE BEGIN RTC_Init 1 */

  /* USER CODE END RTC_Init 1 */
  /** Initialize RTC Only 
  */
  hrtc.Instance = RTC;
  hrtc.Init.HourFormat = RTC_HOURFORMAT_24;
  hrtc.Init.AsynchPrediv = 127;
  hrtc.Init.SynchPrediv = 255;
  hrtc.Init.OutPut = RTC_OUTPUT_DISABLE;
  hrtc.Init.OutPutRemap = RTC_OUTPUT_REMAP_NONE;
  hrtc.Init.OutPutPolarity = RTC_OUTPUT_POLARITY_HIGH;
  hrtc.Init.OutPutType = RTC_OUTPUT_TYPE_OPENDRAIN;
  if (HAL_RTC_Init(&hrtc) != HAL_OK)
  {
    Error_Handler();
  }

  /* USER CODE BEGIN Check_RTC_BKUP */
    
  /* USER CODE END Check_RTC_BKUP */

  /** Initialize RTC and set the Time and Date 
  */
  sTime.Hours = 0;
  sTime.Minutes = 0;
  sTime.Seconds = 0;
  sTime.DayLightSaving = RTC_DAYLIGHTSAVING_NONE;
  sTime.StoreOperation = RTC_STOREOPERATION_RESET;
  if (HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BIN) != HAL_OK)
  {
    Error_Handler();
  }
  sDate.WeekDay = RTC_WEEKDAY_MONDAY;
  sDate.Month = RTC_MONTH_JANUARY;
  sDate.Date = 1;
  sDate.Year = 0;

  if (HAL_RTC_SetDate(&hrtc, &sDate, RTC_FORMAT_BIN) != HAL_OK)
  {
    Error_Handler();
  }
  /** Enable the WakeUp 
  */
  if (HAL_RTCEx_SetWakeUpTimer_IT(&hrtc, 720, RTC_WAKEUPCLOCK_CK_SPRE_16BITS) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN RTC_Init 2 */

  /* USER CODE END RTC_Init 2 */

}

/**
  * @brief TIM21 Initialization Function
  * @param None
  * @retval None
  */
static void MX_TIM21_Init(void)
{

  /* USER CODE BEGIN TIM21_Init 0 */

  /* USER CODE END TIM21_Init 0 */

  TIM_ClockConfigTypeDef sClockSourceConfig = {0};
  TIM_MasterConfigTypeDef sMasterConfig = {0};
  TIM_OC_InitTypeDef sConfigOC = {0};

  /* USER CODE BEGIN TIM21_Init 1 */

  /* USER CODE END TIM21_Init 1 */
  htim21.Instance = TIM21;
  htim21.Init.Prescaler = 2097;
  htim21.Init.CounterMode = TIM_COUNTERMODE_UP;
  htim21.Init.Period = 999;
  htim21.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
  htim21.Init.AutoReloadPreload = TIM_AUTORELOAD_PRELOAD_DISABLE;
  if (HAL_TIM_Base_Init(&htim21) != HAL_OK)
  {
    Error_Handler();
  }
  sClockSourceConfig.ClockSource = TIM_CLOCKSOURCE_INTERNAL;
  if (HAL_TIM_ConfigClockSource(&htim21, &sClockSourceConfig) != HAL_OK)
  {
    Error_Handler();
  }
  if (HAL_TIM_PWM_Init(&htim21) != HAL_OK)
  {
    Error_Handler();
  }
  sMasterConfig.MasterOutputTrigger = TIM_TRGO_RESET;
  sMasterConfig.MasterSlaveMode = TIM_MASTERSLAVEMODE_DISABLE;
  if (HAL_TIMEx_MasterConfigSynchronization(&htim21, &sMasterConfig) != HAL_OK)
  {
    Error_Handler();
  }
  sConfigOC.OCMode = TIM_OCMODE_PWM1;
  sConfigOC.Pulse = 0;
  sConfigOC.OCPolarity = TIM_OCPOLARITY_HIGH;
  sConfigOC.OCFastMode = TIM_OCFAST_DISABLE;
  if (HAL_TIM_PWM_ConfigChannel(&htim21, &sConfigOC, TIM_CHANNEL_1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN TIM21_Init 2 */

  /* USER CODE END TIM21_Init 2 */

}

/**
  * @brief USART1 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART1_UART_Init(void)
{

  /* USER CODE BEGIN USART1_Init 0 */

  /* USER CODE END USART1_Init 0 */

  /* USER CODE BEGIN USART1_Init 1 */

  /* USER CODE END USART1_Init 1 */
  huart1.Instance = USART1;
  huart1.Init.BaudRate = 9600;
  huart1.Init.WordLength = UART_WORDLENGTH_8B;
  huart1.Init.StopBits = UART_STOPBITS_1;
  huart1.Init.Parity = UART_PARITY_NONE;
  huart1.Init.Mode = UART_MODE_TX_RX;
  huart1.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart1.Init.OverSampling = UART_OVERSAMPLING_16;
  huart1.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart1.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart1) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART1_Init 2 */

  /* USER CODE END USART1_Init 2 */

}

/**
  * @brief USART2 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART2_UART_Init(void)
{

  /* USER CODE BEGIN USART2_Init 0 */

  /* USER CODE END USART2_Init 0 */

  /* USER CODE BEGIN USART2_Init 1 */

  /* USER CODE END USART2_Init 1 */
  huart2.Instance = USART2;
  huart2.Init.BaudRate = 9600;
  huart2.Init.WordLength = UART_WORDLENGTH_8B;
  huart2.Init.StopBits = UART_STOPBITS_1;
  huart2.Init.Parity = UART_PARITY_NONE;
  huart2.Init.Mode = UART_MODE_TX_RX;
  huart2.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart2.Init.OverSampling = UART_OVERSAMPLING_16;
  huart2.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart2.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart2) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART2_Init 2 */

  /* USER CODE END USART2_Init 2 */

}

/**
  * @brief USART5 Initialization Function
  * @param None
  * @retval None
  */
static void MX_USART5_UART_Init(void)
{

  /* USER CODE BEGIN USART5_Init 0 */

  /* USER CODE END USART5_Init 0 */

  /* USER CODE BEGIN USART5_Init 1 */

  /* USER CODE END USART5_Init 1 */
  huart5.Instance = USART5;
  huart5.Init.BaudRate = 19200;
  huart5.Init.WordLength = UART_WORDLENGTH_8B;
  huart5.Init.StopBits = UART_STOPBITS_1;
  huart5.Init.Parity = UART_PARITY_NONE;
  huart5.Init.Mode = UART_MODE_TX_RX;
  huart5.Init.HwFlowCtl = UART_HWCONTROL_NONE;
  huart5.Init.OverSampling = UART_OVERSAMPLING_16;
  huart5.Init.OneBitSampling = UART_ONE_BIT_SAMPLE_DISABLE;
  huart5.AdvancedInit.AdvFeatureInit = UART_ADVFEATURE_NO_INIT;
  if (HAL_UART_Init(&huart5) != HAL_OK)
  {
    Error_Handler();
  }
  /* USER CODE BEGIN USART5_Init 2 */

  /* USER CODE END USART5_Init 2 */

}

/**
  * @brief GPIO Initialization Function
  * @param None
  * @retval None
  */
static void MX_GPIO_Init(void)
{
  GPIO_InitTypeDef GPIO_InitStruct = {0};

  /* GPIO Ports Clock Enable */
  __HAL_RCC_GPIOC_CLK_ENABLE();
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();

  /*Configure GPIO pin Output Level */
  HAL_GPIO_WritePin(GPIOA, EN_GPS_Pin|EN_Part_Pin|Sleep_Fox_Pin, GPIO_PIN_RESET);

  /*Configure GPIO pins : EN_GPS_Pin EN_Part_Pin Sleep_Fox_Pin */
  GPIO_InitStruct.Pin = EN_GPS_Pin|EN_Part_Pin|Sleep_Fox_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_LOW;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  /*Configure GPIO pin : Wind_Speed_Pin */
  GPIO_InitStruct.Pin = Wind_Speed_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_IT_RISING;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  HAL_GPIO_Init(Wind_Speed_GPIO_Port, &GPIO_InitStruct);

  /*Configure GPIO pin : INT_MPL_Pin */
  GPIO_InitStruct.Pin = INT_MPL_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_INPUT;
  GPIO_InitStruct.Pull = GPIO_NOPULL;
  HAL_GPIO_Init(INT_MPL_GPIO_Port, &GPIO_InitStruct);

  /* EXTI interrupt init*/
  HAL_NVIC_SetPriority(EXTI4_15_IRQn, 0, 0);
  HAL_NVIC_EnableIRQ(EXTI4_15_IRQn);

}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler_Debug */
  /* User can add his own implementation to report the HAL error return state */

  /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN 6 */
  /* User can add his own implementation to report the file name and line number,
     tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
